/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/* This unit implements functions related users contacts list processing  */
/*									  */
/* Contributor:	Alexandr V. Shutko <AVShutko@mail.khstu.ru>		  */
/* Created:	15 November 2000					  */
/* Updated:	26 December 2000 by Regressor <AVShutko@mail.khstu.ru>	  */
/*									  */
/**************************************************************************/

#include "includes.h"

/**************************************************************************/
/* Insert new contact list in contact database 				  */
/* This function do not delete allocated memory, we should do it after    */
/* function call 							  */
/**************************************************************************/
int db_contact_insert(unsigned long uin, int number, 
		      unsigned long *contact, int type)
{
   PGresult *res;
   
   cstring dbcomm_str;
   
   DEBUG(100, ("Inserting %d records for %lu to contact database.\n", 
            number, uin));
   
   for (int i=0; i<number; i++)
   {
      slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
              "INSERT INTO Online_Contacts values (%d, %d, %d)", 
	      uin, contact[i], type);

      res = PQexec(users_dbconn, dbcomm_str);
      if (PQresultStatus(res) != PGRES_COMMAND_OK)
      {
         LOG_SYS(0, ("SQL Command failed...\n"));
         LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
         PQclear(res);
         return(-1);
      }
      else
      {
         PQclear(res);
      }
   }
   return(0);
}

/**************************************************************************/
/* Delete contact records with specified owner from database		  */
/**************************************************************************/
int db_contact_delete(unsigned long uin)
{
   PGresult *res;
   
   fstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "DELETE FROM Online_Contacts WHERE ouin=%d", uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}

/**************************************************************************/
/* Lookup all users who should be alerted if user with specified	  */
/* uin change status. This function return number of retrieved users	  */
/* also it alloc memory for returned list, "contact" pointer should be 	  */
/* NULL to avoid memory leak 						  */
/**************************************************************************/
int db_contact_lookup(unsigned long uin, int type, unsigned long **contact)
{
   PGresult *res;
   int number;

   fstring dbcomm_str;
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "SELECT ouin FROM Online_Contacts WHERE tuin=%d AND type=%d", 
	   uin, type);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(0);
   }

   number = PQntuples(res);
   
   if (number > 0)
   {
      if (PQnfields(res) < 1) 
      {
         LOG_SYS(0, ("Corrypted table structure in db: \"%s\"\n", 
	             lp_db_users()));
         exit(-1);
      }
      
      *contact = (unsigned long *)malloc(sizeof(unsigned long)*number);
      DEBUG(200, ("Trying to malloc %d bytes for %d records (result: %06X)\n",
                  (sizeof(unsigned long)*number), number, 
		  (unsigned int)*contact));
		
      for (int i=0;i<number;i++) (*contact)[i] = atol(PQgetvalue(res, i, 0));	

      PQclear(res);
      return(number);
   }
   else
   {
      PQclear(res);  
      return(0);
   }
}
