/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/* This file contain definitions for all database functions (i.e. user	  */
/* info structures, online structures)					  */
/*									  */
/* Contributor:	Alexandr V. Shutko <AVShutko@mail.khstu.ru>		  */
/* Created:	07 June 2000					  	  */
/* Updated:	26 December 2000 by Regressor <AVShutko@mail.khstu.ru>	  */
/*									  */
/**************************************************************************/

#define ONLINE_DB_FNUM 10	/* num of fields in online db 		  */
#define USERS_DB_FNUM 46	/* num of fields in users db 		  */
#define DEPS_MAX_NUM 250	/* max number of departments 		  */

/* types of contact lists (for contact database) */
#define NORMAL_CONTACT     1	/* normal contact list  		  */
#define INVISIBLE_CONTACT  2	/* invisible to users   		  */
#define VISIBLE_CONTACT    3	/* visible to users     		  */
#define IGNORE_CONTACT     4	/* user's ignore list   		  */

/* variables for set web page function */
#define HOME  1
#define WORK  2

/* This is single record for online database */
typedef struct online_user
{
  unsigned long  uin;		/* online user uin number 		  */
  in_addr  	 ip;		/* user's ip 				  */
  unsigned long  udp_port;	/* user's back udp port 		  */
  unsigned long  tcp_port;	/* user's tcp server port 		  */
  unsigned long  status;	/* user current status 			  */
  unsigned long  uptime;	/* last login time 			  */
  unsigned long  lutime;	/* last update time 			  */
  unsigned long  ttl;		/* max live-time without update 	  */
  unsigned short protocol;	/* protocol number 			  */
  unsigned long  servseq;	/* current server sequence num		  */
  unsigned long  servseq2;	/* v5 proto server sequence 2		  */
  unsigned long  session_id;	/* v5 session random number (id)	  */
  unsigned short state;		/* client state 			  */
  unsigned short tcpver;	/* client TCP protocol version		  */
   
} online_user;


/* single record for users info database (full version) */
/* i have know idea where it should be used exept database convertation   */

typedef struct full_user_info
{
    unsigned long        uin; 	/* user uin number 			  */
    short	     pemail1;   /* if email1 is public 			  */
    short	        auth;   /* authorization 			  */
    short                age;	/* user's age 				  */
    short             gender;	/* user's gender 			  */
    short	    disabled;   /* if user's account disabled 		  */
    short      can_broadcast;   /* if he can send broadcast messages 	  */
    int          ch_password;   /* if user need to change password 	  */
    int 	  gmt_offset;   /* user gmt time offset 		  */
    
    short             bmonth;   /* user's birth day 			  */
    short      		bday;   /* user's bith month 			  */
    short     	       byear;   /* user's birth year 			  */
    long 	        wzip;   /* work cell phone 			  */
    short 	       wocup;   /* work ocupation 			  */
    long 	     wdepart;   /* work department 			  */
    short	    wcountry;	/* work country 			  */  
    
    long    	     cr_date;   /* date of account creation 		  */
    long  	   lastlogin;   /* last user login time 		  */
    unsigned long    ip_addr;   /* last user's ip address 		  */
    unsigned long    nupdate;   /* notes update time 			  */
    
    short	    hcountry;	/* home country 			  */
    long 	        hzip;   /* home cell phone 			  */

    char          hpage[128];   /* www home page 			  */
    char          hphone[32];	/* home phone 				  */
    char 	    hfax[32];   /* home fax 				  */
    char 	   hcell[32];   /* home cell phone 			  */    

    char	   waddr[64];   /* work address 			  */
    char           wcity[32];	/* work city 				  */
    char          wstate[32];	/* work state 				  */  
    char 	wcompany[32];   /* work company name 			  */
    char 	  wtitle[32];   /* work title 				  */
    char          wphone[32];	/* work phone 				  */
    char 	  wpager[32];   /* work cell phone 			  */        
    char 	    wfax[32];   /* work fax 				  */    
    char 	  wpage[128];	/* work www page 			  */
    char          passwd[32];	/* user's password 			  */
    char            nick[32];	/* user's nickname 			  */
    char            last[32]; 	/* user's lastname 			  */
    char           first[32]; 	/* user's firstname 			  */
    char          email1[64]; 	/* user's first email addres 		  */
    char          email2[64]; 	/* user's second email addres 		  */
    char          email3[64]; 	/* user's third email addres 		  */
    char	   haddr[64];   /* home address 			  */
    char           hcity[32];	/* home city 				  */
    char          hstate[32];	/* home state 				  */

    char          notes[255];	/* users notes 				  */
    
} full_user_info;

/* partial user information - used in login */
typedef struct login_user_info
{
    unsigned long        uin; 	/* user uin number 			  */
    int		     pemail1;   /* if email1 is public 			  */
    int		    disabled;   /* if user's account disabled 		  */
    int        can_broadcast;   /* if he can send broadcast messages 	  */
    int          ch_password;   /* if user need to change password 	  */
    char          passwd[32];	/* user's password 			  */

} login_user_info;


/* partial user information - used in send notes */
typedef struct notes_user_info
{
    unsigned long 	 uin;	/* notes owner uin number 		  */
    unsigned long    nupdate;   /* notes last update time 		  */
    unsigned long  lastlogin;   /* user last login time 		  */
    unsigned long    ip_addr;   /* user's last ip addr 			  */
    char          notes[255];	/* user's notes 			  */

} notes_user_info;

/* information about single department */
typedef struct depart_info
{
    int		     depcode;	/* department unique code 		  */
    char           depmin[8];	/* department short name 		  */
    char	depname[255];   /* department full name 		  */
    
} depart_info;


/* information about found user */
typedef struct found_info
{
    unsigned long        uin; 	/* user uin number 			  */
    int		     pemail1;   /* if email1 is public 			  */
    int		        auth;   /* authorization 			  */
    char            nick[32];	/* user's nickname 			  */
    char            last[32]; 	/* user's lastname 			  */
    char           first[32]; 	/* user's firstname 			  */
    char          email1[64]; 	/* user's first email addres 		  */
    char          email2[64]; 	/* user's second email addres 		  */
    char          email3[64]; 	/* user's third email addres 		  */
    
} found_info;


