/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/* This file contain definitions for database table create-queries and 	  */
/* table parameters for automatic database initialization and checks 	  */
/*									  */
/* Contributor:	Alexandr V. Shutko <AVShutko@mail.khstu.ru>		  */
/* Created:	12 September 2000					  */
/* Updated:	26 December 2000 by Regressor <AVShutko@mail.khstu.ru>	  */
/*									  */
/**************************************************************************/

/* index in check_result array */
#define DB_EXIST 	  0

/* online contacts table initialization data */
#define CONT_TBL_OK	  1
#define CONT_TBL_FIELDS   3
#define CONT_TBL_NAME     "online_contacts"
#define CONT_TBL_CREATE	  "CREATE TABLE Online_Contacts (ouin int4, tuin int4, type int2); \
			   CREATE INDEX onln_ouin ON Online_Contacts (ouin); \
			   CREATE INDEX onln_tuin ON Online_Contacts (tuin);"

/* departments table initialization data */
#define DEPS_TBL_OK	  2
#define DEPS_TBL_FIELDS   3
#define DEPS_TBL_NAME     "users_deps"
#define DEPS_TBL_CREATE	  "CREATE TABLE Users_Deps (depcode int4, depmin name, depname text); \
		           CREATE UNIQUE INDEX depcode ON Users_Deps (depcode);"

/* fragment storage table initialization data */
#define DFRG_TBL_OK	  3
#define DFRG_TBL_FIELDS   6
#define DFRG_TBL_NAME     "fragment_storage"
#define DFRG_TBL_CREATE   "CREATE TABLE Fragment_storage (uin int4, seq int4, prt_num int2, \
    			   prt_cnt int2, len int2, frg int4); \
		           CREATE INDEX frg_uin ON Fragment_storage (uin); \
		           CREATE INDEX frg_prt ON Fragment_storage (prt_num); \
		           CREATE INDEX frg_seq ON Fragment_storage (seq);"

/* locks table initialization data */
#define LOCK_TBL_OK	  4
#define LOCK_TBL_FIELDS   2
#define LOCK_TBL_NAME     "users_lock"
#define LOCK_TBL_CREATE   "CREATE TABLE Users_Lock (luin int4, lck_text text); \
		           CREATE UNIQUE INDEX lock_uin ON Users_Lock (luin); "

/* messages table initialization data */
#define MESS_TBL_OK	  5
#define MESS_TBL_FIELDS   5
#define MESS_TBL_NAME     "users_messages"
#define MESS_TBL_CREATE   "CREATE TABLE Users_Messages (to_uin int4, from_uin int4, msg_date int4, \
		           msg_type int2, msgtext text); \
		           CREATE INDEX mess_from_uin ON Users_Messages (from_uin); \
		           CREATE INDEX mess_to_uin ON Users_Messages (to_uin); \
		           CREATE INDEX mess_msg_date ON Users_Messages (msg_date);"

/* online_users table initialization data */
#define ONLN_TBL_OK	  6
#define ONLN_TBL_FIELDS   14
#define ONLN_TBL_NAME     "online_users"
#define ONLN_TBL_CREATE   "CREATE TABLE Online_Users (uin int4, iadr int4, uprt int4, \
		           tprt int4, stat int4, uptm int4, lutm int4, ttl int4, \
		           prv int2, sseq int4, sseq2 int4, sess_id int4, state int2, \
			   tcpver int2); \
		           CREATE UNIQUE INDEX onln_uin ON Online_Users (uin);"

/* users table initialization data */
#define USER_TBL_OK	  7
#define USER_TBL_FIELDS   46
#define USER_TBL_NAME     "users_info"
#define USER_TBL_CREATE   "CREATE TABLE Users_Info (uin int4, pass name, ulock int2, llog int4, \
		           iadr int8, bcst int2, cdate int4, cpass int2, nick name, frst name, \
		           last name, email1 text, email2 text, email3 text, e1publ int2, \
		           gmtoffs int2, auth int2, sex int2, age int2, bday int2, bmon int2, \
		           byear int2, waddr text, wcity name, wstate name, wcountry int4, \
		           wcompany name, wtitle name, wocup int2, wdepart name, wphon name, \
		           wfax name, wpager name, wzip text, wweb text, notes text, haddr text, \
		           hcity name, hstate name, hcountry int4, hphon name, hfax name, \
		           hcell name, hzip name, hweb text, nnotes int4); "
#define USER_TBL_CREATE1  "CREATE UNIQUE INDEX user_uin ON Users_Info (uin); \
			   CREATE INDEX user_last ON Users_Info (last); \
		           CREATE INDEX user_frst ON Users_Info (frst); \
		           CREATE INDEX user_nick ON Users_Info (nick); \
		           CREATE INDEX user_iadr ON Users_Info (iadr); \
		           CREATE INDEX user_llog ON Users_Info (llog); \
		           CREATE VIEW Users AS SELECT uin, frst, last, bcst, pass FROM Users_Info;"
