/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/* This unit implements online database calls (lookup, sseq control, etc) */
/* for packet processors						  */
/*									  */
/* Contributor:	Alexandr V. Shutko <AVShutko@mail.khstu.ru>		  */
/* Created:	02 September 2000					  */
/* Updated:	26 December 2000 by Regressor <AVShutko@mail.khstu.ru>	  */
/*									  */
/**************************************************************************/

#include "includes.h"

/**************************************************************************/
/* This func used to set server sequence to specified value for uin	  */
/**************************************************************************/
int db_online_update_servseq(struct online_user &temp_user)
{
   PGresult *res;
   
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "UPDATE Online_Users SET sseq=%d WHERE uin=%d", 
	    temp_user.servseq, temp_user.uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}

/**************************************************************************/
/* This func will add specified number to server sequence		  */
/**************************************************************************/
int db_online_add_servseq(unsigned long uin, int number)
{
   PGresult *res;
   
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "UPDATE Online_Users SET sseq=sseq+%d WHERE uin=%u", 
	    number, uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}

/**************************************************************************/
/* This func will update user's status			      		  */
/**************************************************************************/
int db_online_setstatus(struct online_user &temp_user)
{
   PGresult *res;
   
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "UPDATE Online_Users SET stat=%d WHERE uin=%d", 
	    temp_user.status, temp_user.uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }

}

/**************************************************************************/
/* This func update user's client state			      		  */
/**************************************************************************/
int db_online_setstate(struct online_user &temp_user, int state)
{
   PGresult *res;
   
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "UPDATE Online_Users SET state=%d WHERE uin=%d", 
	    state, temp_user.uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}


/**************************************************************************/
/* This func will update user's last update time	      		  */
/**************************************************************************/
int db_online_touch(struct online_user &temp_user)
{
   PGresult *res;
   
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "UPDATE Online_Users SET lutm=%d WHERE uin=%d", 
	    timeToLong(time(NULL)), temp_user.uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}

/**************************************************************************/
/* This function insert new user record to online database	 	  */
/**************************************************************************/
int  db_online_insert(struct online_user &temp_user)
{
   PGresult *res;
   cstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "INSERT INTO Online_Users values (%d, %d, %d, %d, %d, %d, 
	    %d, %d, %d, %d, %d, %d, 255, %d)", temp_user.uin, 
            ipToIcq(temp_user.ip), temp_user.udp_port, temp_user.tcp_port, 
            temp_user.status, temp_user.uptime, temp_user.lutime, 
            temp_user.ttl, temp_user.protocol, temp_user.servseq, 
            temp_user.servseq2, temp_user.session_id, temp_user.tcpver);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);  
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
  
}

/**************************************************************************/
/* This function delete user record from online database	 	  */
/**************************************************************************/
int  db_online_delete(unsigned long to_uin)
{
   PGresult *res;
   
   fstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "DELETE FROM Online_Users WHERE uin=%d", to_uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQcmdTuples(res) != "")
   {
      PQclear(res);
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);    
   }
}

/**************************************************************************/
/* This function return user data from online database			  */
/**************************************************************************/
int db_online_lookup(unsigned long to_uin, struct online_user &temp_user)
{
   PGresult *res;
   
   fstring dbcomm_str;
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "SELECT * FROM Online_Users WHERE uin=%d", to_uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQnfields(res) != ONLN_TBL_FIELDS) 
   {
      LOG_SYS(0, ("Corrypted table structure in online_table: \n"));
      exit(-1);
   }
   
   if (PQntuples(res) > 0)
   {
      temp_user.uin		= atol(PQgetvalue(res, 0, 0));
      temp_user.ip		= icqToIp(atol(PQgetvalue(res, 0, 1))); 
      temp_user.udp_port	= atol(PQgetvalue(res, 0,  2)); 
      temp_user.tcp_port	= atol(PQgetvalue(res, 0,  3)); 
      temp_user.status		= atol(PQgetvalue(res, 0,  4));
      temp_user.uptime		= atol(PQgetvalue(res, 0,  5)); 
      temp_user.lutime		= atol(PQgetvalue(res, 0,  6)); 
      temp_user.ttl		= atol(PQgetvalue(res, 0,  7)); 
      temp_user.protocol	= (unsigned short)atoi(PQgetvalue(res, 0, 8)); 
      temp_user.servseq		= atol(PQgetvalue(res, 0,  9));
      temp_user.servseq2	= atol(PQgetvalue(res, 0, 10));
      temp_user.session_id	= atol(PQgetvalue(res, 0, 11));
      temp_user.state		= atol(PQgetvalue(res, 0, 12));
      temp_user.tcpver		= atol(PQgetvalue(res, 0, 13)); 
    
      PQclear(res);  
      return(0);
      
   }
   else 
   {
      PQclear(res);
      return(-1);
   }
}

/**************************************************************************/
/* This function return user data from online database			  */
/**************************************************************************/
int db_online_lookup(unsigned long to_uin, 
		     struct online_user &temp_user,
		     unsigned short magnum1,
		     unsigned short magnum2)
{
   PGresult *res;
   fstring dbcomm_str;
   
   if ((to_uin == ADMIN_UIN) &&
       (ipc_vars->magic_num1 == magnum1) &&
       (ipc_vars->magic_num2 == magnum2))
   {
      temp_user.uin		= ADMIN_UIN;
      temp_user.ip		= icqToIp(INADDR_LOOPBACK);;
      temp_user.udp_port	= 4000;
      temp_user.tcp_port	= 4000;
      temp_user.status		= 0;
      temp_user.uptime		= 0;
      temp_user.lutime		= 0;
      temp_user.ttl		= 0;
      temp_user.protocol	= V3_PROTO;
      temp_user.servseq		= 0;
      temp_user.tcpver		= 0;
	
      ipc_vars->magic_num1 = random_num();
      ipc_vars->magic_num2 = random_num();
      
      return(0);   
   }
   
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "SELECT * FROM Online_Users WHERE uin=%d", to_uin);

   res = PQexec(users_dbconn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQnfields(res) != ONLN_TBL_FIELDS) 
   {
      LOG_SYS(0, ("Corrypted table structure in online_table: \n"));
      exit(-1);
   }
   
   if (PQntuples(res) > 0)
   {
      temp_user.uin		= atol(PQgetvalue(res, 0, 0));
      temp_user.ip		= icqToIp(atol(PQgetvalue(res, 0, 1))); 
      temp_user.udp_port	= atol(PQgetvalue(res, 0,  2)); 
      temp_user.tcp_port	= atol(PQgetvalue(res, 0,  3)); 
      temp_user.status		= atol(PQgetvalue(res, 0,  4));
      temp_user.uptime		= atol(PQgetvalue(res, 0,  5)); 
      temp_user.lutime		= atol(PQgetvalue(res, 0,  6)); 
      temp_user.ttl		= atol(PQgetvalue(res, 0,  7)); 
      temp_user.protocol	= (unsigned short)atoi(PQgetvalue(res, 0, 8)); 
      temp_user.servseq		= atol(PQgetvalue(res, 0,  9));
      temp_user.servseq2	= atol(PQgetvalue(res, 0, 10));
      temp_user.session_id	= atol(PQgetvalue(res, 0, 11));
      temp_user.state		= atol(PQgetvalue(res, 0, 12));
      temp_user.tcpver		= atol(PQgetvalue(res, 0, 13)); 
    
      PQclear(res);  
      return(0);
      
   }
   else 
   {
      PQclear(res);
      return(-1);
   }
}


/**************************************************************************/
/* This function return user data from online database and inc sseq 	  */
/**************************************************************************/
int db_online_sseq_open(struct online_user &temp_user)
{
   PGresult *res;

   fstring dbcomm_str;
   /* exec select command on backend server */
   
   PQclear( PQexec(users_dbconn, "BEGIN" ));
   slprintf(dbcomm_str, 1023, "SELECT sseq FROM Online_Users WHERE uin=%d 
            FOR UPDATE", temp_user.uin);

   res = PQexec(users_dbconn, dbcomm_str);
   
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      LOG_SYS(0, ("SQL Command failed...\n"));
      LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
      PQclear(res);
      return(-1);
   }

   if (PQntuples(res) > 0) 
   {
      temp_user.servseq = atol(PQgetvalue(res, 0, 0));
      PQclear(res);  
      
      return(0);
   }
   else 
   {
      PQclear(res);
      return(-1);
   }

}


/**************************************************************************/
/* This function return user data from online database and inc sseq 	  */
/**************************************************************************/
int db_online_sseq_close(struct online_user &temp_user, int inc_num)
{
   PGresult *res;
   fstring dbcomm_str;
   
   /* exec update command on backend server */
   slprintf(dbcomm_str, 1023, "UPDATE Online_Users SET sseq=sseq+%d 
            WHERE uin=%d", inc_num, temp_user.uin);
	    
   res = PQexec(users_dbconn, dbcomm_str);
   PQclear(res);
   
   PQclear( PQexec(users_dbconn, "END" ));
   return(0);

}

/**************************************************************************/
/* I DON'T WANT RECEIVE SPAM SHIT NOTICE MESSAGES FROM BACKEND!!!!!	  */
/* I lost whole day trying understand why libpq send spam to server       */
/* incoming IPC pipe and then found that pipe after closing stderr        */
/* receive the SAME number and foolish libpq notice processor             */
/* simply make printf(stderr, bla-bla-bla) to server pipe!!!	          */
/**************************************************************************/
void oNoticeStub(void * arg, const char * message)
{
   return;
}
