/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "icontrol.h"

extern class TPCollection *server_plist;
extern class TPListDialog *plistdlg;
extern BOOL plist_refresh;
extern otstream *term_in;

/* ============ TPListDialog class =============*/
TPListDialog::TPListDialog( TPCollection *plist) :
    TDialog( TRect( 15, 2, 60, 13 ), "Processes list" ),
    TWindowInit( &TPListDialog::initFrame )
{
    options |= ofCentered;

    TScrollBar *sb = new TScrollBar( TRect( 28, 2, 29, 9 ) );
    insert( sb );
    PList = new TPListBox( TRect( 2, 2, 28, 9 ), sb );
    PList->showProcesses(plist);
    insert( PList );

    KillButton    = new TButton( TRect( 29, 2, 43, 3 ), "Sig~K~ill ", cm_PSigkill, bfNormal );
    insert( KillButton );
    CoreButton    = new TButton( TRect( 29, 4, 43, 5 ), "Core~d~ump", cm_PCoredump, bfNormal );
    insert( CoreButton );
    RefreshButton = new TButton( TRect( 29, 6, 43, 7 ), "~R~efresh ", cm_PRefresh, bfNormal );
    insert( RefreshButton );    
    CancelButton  = new TButton( TRect( 29, 8, 43, 9 ), "~C~ancel  ", cmCancel, bfDefault );
    insert( CancelButton );

    plist_refresh = True;
    selectNext( False );
}

void TPListDialog::handleEvent( TEvent& event )
{
    unsigned long  pr_pid;
    unsigned short pr_role;
    
    TDialog::handleEvent( event );
    switch( event.what )
        {
        case evCommand:
            {
            switch( event.message.command )
                {
		case cm_PRefresh:
		    plist_refresh = True;
		    break;
		    
		case cm_PCoredump:
		    plist_refresh = True;
		    pr_pid  = PList->list()->at(PList->focused)->get_proc_pid();
		    pr_role = PList->list()->at(PList->focused)->get_proc_role();
		    kill(pr_pid, SIGABRT);
		    *term_in << "\\> Sending SIGABRT signal to ";
		    *term_in << pr_pid << " (role: " << pr_role << " )\n";
		    break;
		    
		case cm_PSigkill:
		    plist_refresh = True;
		    pr_pid  = PList->list()->at(PList->focused)->get_proc_pid();
		    pr_role = PList->list()->at(PList->focused)->get_proc_role();
		    kill(pr_pid, SIGINT);
		    *term_in << "\\> Sending SIGINT signal to ";
		    *term_in << pr_pid << " (role: " << pr_role << " )\n";
		    break;
		
		case cmCancel:
		    destroy(this);
		    break;
                default:
                    return;
                }
	    }
        default:
            break;
        }
}

TPListDialog::~TPListDialog()
{
  plistdlg = NULL;
}

/* ================  TPListBox class ================*/
TPListBox::TPListBox( const TRect& bounds, TScrollBar *aScrollBar ) :
    TListBox( bounds, 1, aScrollBar )
{
}

TPListBox::~TPListBox()
{
   if ( list() )
      destroy( list() );
}

void TPListBox::getText( char *text, short item, short maxChars )
{
    strncpy( text, list()->at(item)->text(), maxChars );
    text[maxChars] = '\0';
}

Boolean TPListBox::isSelected( short item )
{
    return (True);
}

void TPListBox::showProcesses( TPCollection *procs )
{
	newList(procs);
}

/* ========= TPCollection class ========== */
void TPCollection::writeItem( void *obj, opstream& os )
{
}

void *TPCollection::readItem( ipstream& is )
{
    TPEntry *entry;
    return entry;
}


/* ========= TPEntry class ==========*/
TPEntry::TPEntry( long p_pid, long p_role)
{
  proc_pid  = p_pid;
  proc_role = p_role;

  switch (proc_role)
  {
    case ROLE_SOCKET: snprintf(displayText,29, "[%05ld] Socket processor",
		      proc_pid); break;
    case ROLE_PACKET: snprintf(displayText,29, "[%05ld] Packet processor",
		      proc_pid); break;
    case ROLE_EVENT:  snprintf(displayText,29, "[%05ld] Event processor", 
		      proc_pid); break;
    case ROLE_BUSY:   snprintf(displayText,29, "[%05ld] Busy processor",  
		      proc_pid); break;
    default: 	      snprintf(displayText,29, "[%05ld] Unknown process", 
		      proc_pid); break;
  }
}
