/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#ifndef __TPLISTDIALOG_H
#define __TPLISTDIALOG_H

/* single entry in TPCollection */
class TPEntry
{

public:

    TPEntry( long p_pid, long p_role);
    long get_proc_pid()  { return proc_pid;  }
    long get_proc_role() { return proc_role; }
    char *text() { return (char *)displayText; }
    
private:

    char displayText[30];
    long proc_pid;
    long proc_role;

};

/* processes collection class */
class TPCollection : public TCollection
{

public:

    TPCollection( ccIndex aLimit, ccIndex aDelta) :
        TCollection( aLimit, aDelta ) {}

    TPEntry *at( ccIndex index )
        { return (TPEntry *)TCollection::at( index );}
    virtual ccIndex indexOf( TPEntry *item )
        { return TCollection::indexOf( item ); }

    void remove( TPEntry *item )
        { TCollection::remove( item ); }
    void free( TPEntry *item )
        { TCollection::free( item ); }
    void atInsert( ccIndex index, TPEntry *item )
        { TCollection::atInsert( index, item ); }
    void atPut( ccIndex index, TPEntry *item )
        { TCollection::atPut( index, item ); }
    virtual ccIndex insert( TPEntry *item )
        { return TCollection::insert( item ); }

    TPEntry *firstThat( ccTestFunc Test, void *arg );
    TPEntry *lastThat( ccTestFunc Test, void *arg );

private:

    virtual void freeItem( void *item )
        { delete (TPEntry *)item; }
    virtual void *readItem( ipstream& );
    virtual void writeItem( void *, opstream& );


public:

    static const char * const name;
    static TStreamable *build();

};

inline TPEntry *TPCollection::firstThat( ccTestFunc func, void *arg )
{
    return (TPEntry *)TCollection::firstThat( ccTestFunc(func), arg );
}

inline TPEntry *TPCollection::lastThat( ccTestFunc func, void *arg )
{
    return (TPEntry *)TCollection::lastThat( ccTestFunc(func), arg );
}


/* view with process list */
class TPListBox : public TListBox
{

public:

    TPListBox( const TRect& bounds, TScrollBar *aScrollBar );
    ~TPListBox();

    virtual void getText( char *, short, short );
    virtual Boolean isSelected( short );
    TPCollection *list();

    void showProcesses( TPCollection * );

};

inline TPCollection *TPListBox::list()
{
    return (TPCollection *)TListBox::list();
}


/* This dialog show processes list and allow kill 
   or hup then with buttons */
   
class TPListDialog : public TDialog
{

public:

    TPListDialog( TPCollection *plist );
    ~TPListDialog();
    virtual void handleEvent( TEvent& );

    TPListBox *PList;
    TButton *CoreButton;
    TButton *KillButton;
    TButton *CancelButton;
    TButton *RefreshButton;

};


#endif /* __TPLISTDIALOG_H */
