/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "icontrol.h"

/* Class TInfoView */

TInfoView::TInfoView( const TRect& bounds ) : TView( bounds )
{
   growMode = 0;
   eventMask = evMouseDown | evKeyDown | evCommand | evBroadcast;
  
   info_data.lutime = time(NULL) - 10;
}

void TWinInfo::set_info_data(struct TServinfo &sinfo)
{
  info_view->info_data = sinfo;
}


void TWinInfo::get_info_data(struct TServinfo &sinfo)
{
  sinfo = info_view->info_data;
}



void TWinInfo::update()
{
  info_view->draw();
}


void TInfoView::draw()
{
    TDrawBuffer buf;
    char chtemp[18];
    
    char c  = getColor(0x0201);
    char c1 = getColor(0x020d);
    char c2 = getColor(0x0201);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Server status: ",    c);  
    
    if ((time(NULL) - info_data.lutime) < 5) 
        buf.moveStr(20, "Alive", getColor(0x217)); 
    else buf.moveStr(20, "Offline", c2); 
    writeLine(0, 0, (short)size.x, 1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Bind address: ",     c);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, inet_ntoa(icqToIp(info_data.bind_addr)), c); 
    else buf.moveStr(20, "Unknown", c1);
    writeLine(0, 1, (short)size.x, 1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Bind port: ",        c);
    snprintf(chtemp, 17,"%lu", info_data.bind_port);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 2, (short)size.x, 1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Server started: ",   c);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, time2str1(info_data.server_started), c); 
    else buf.moveStr(20, "Unknown", c1);
    writeLine(0, 3, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Config loaded: ",    c);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, time2str1(info_data.config_loaded), c); 
    else buf.moveStr(20, "Unknown", c1);
    writeLine(0, 4, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " ----------------- ", c);  
    writeLine(0, 5, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Pack processed: ",   c);
    snprintf(chtemp, 17, "%lu", info_data.pack_processed);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 6, (short)size.x, 1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Pack in queue: ",    c); 
    snprintf(chtemp, 17, "%lu", info_data.pack_in_queue);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 7, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Queue size: ",       c);
    snprintf(chtemp, 17, "%lu b", info_data.queue_size);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 8, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " Used memory: ",      c);
    snprintf(chtemp, 17, "%lu b", info_data.used_memory);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 9, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " ----------------- ", c);
    writeLine(0, 10, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " SP (main)  pid: ",   c);
    snprintf(chtemp, 17, "%lu b", info_data.sp_pid);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 11, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " EP (event) pid: ",   c);
    snprintf(chtemp, 17, "%lu b", info_data.ep_pid);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1);
    writeLine(0, 12, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " BP (busy) num: ",    c); 
    snprintf(chtemp, 17, "%lu b", info_data.bp_num);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 13, (short)size.x,  1, buf);

    buf.moveChar(0, ' ', c, (short)size.x);
    buf.moveStr(0, " PP (pack) num: ",    c);
    snprintf(chtemp, 17, "%lu b", info_data.pp_num);
    if ((time(NULL) - info_data.lutime) < 5) 
       buf.moveStr(20, chtemp, c); else buf.moveStr(20, "Unknown", c1); 
    writeLine(0, 14, (short)size.x,  1, buf);
}

/* Class TWinInfo */

TWinInfo::TWinInfo( const TRect& bounds, const char *aTitle ) :
    TWindow( bounds, aTitle, wnNoNumber ),
    TWindowInit( &TWinInfo::initFrame )
{
   growMode = 0;
   flags = wfMove | wfClose;
   options |= ofSelectable;
   options |= ofWinInfo;
   eventMask = evMouseDown | evKeyDown | evCommand | evBroadcast;
   palette = dpGrayWInfo;

   TRect r( getExtent() );
   r.grow(-1, -1);
   info_view = new TInfoView( r );
   insert(info_view);
}

TPalette& TWinInfo::getPalette() const
{
    static TPalette paletteGray( cpGrayWInfo, sizeof( cpGrayWInfo )-1 );
    return paletteGray;
}

void TWinInfo::handleEvent(TEvent& event)
{
    TWindow::handleEvent(event);
    switch (event.what)
        {
        case evKeyDown:
            switch (event.keyDown.keyCode)
                {
		case kbF10:
                case kbEsc:
		      TWindow::close();
                      clearEvent(event);
                    break;
                }
            break;
        }
}


