/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "icontrol.h"

extern otstream *term_in;

BOOL TMyApp::db_recreate(char *dbname, char *dbuser, char *dbpass, 
		        char *dbaddr, char *dbport)

{
   PGconn   *check_conn;
   PGresult *res;
   BOOL check_result[TABLES_NUMBER];
   cstring dbcomm_str;

   *term_in << "\\> Preparing to purge and recreate database\n";

   if (messageBox( "This operation will destroy all your data...",
       mfWarning | mfCancelButton | mfOKButton) == cmCancel)
   {
       *term_in << "\\> Process canceled by operator...\n";
       return(False);
   }
   
   *term_in << "\\> Checking if database" << dbname << " exist\n";
   /* database template1 connect */
   char	    *pgoptions, *pgtty;

   pgoptions = NULL;	/* special options for backend server */
   pgtty     = NULL;	/* debugging tty for the backend server */

   /* check if specified database exist & mark result */
   /* make a connection to the database */
   check_conn = PQsetdbLogin(dbaddr, dbport, pgoptions, pgtty, 
                                "template1", dbuser, dbpass);

   if (PQstatus(check_conn) == CONNECTION_BAD)
   {
      *term_in << "\\> Can't check database... Connection failed.\n";
      *term_in << PQerrorMessage(check_conn);
      return(False);
   }
	
   PQsetNoticeProcessor(check_conn, 
   (void (*)(void *, const char *))uNoticeStub, NULL);
    
   /* execute select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "SELECT COUNT(*) FROM pg_database WHERE datname like '%s'", dbname);

   res = PQexec(check_conn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      *term_in << "\\> SQL Command failed...\n\\> " << PQresultErrorMessage(res);
      PQclear(res);
      return (False);
   }
   
   if (PQntuples(res) > 0) 
   {
      if (atoi(PQgetvalue(res, 0, 0)) > 0) 
      {
         check_result[DB_EXIST] = True;
	 /* now we can delete database - it exist */
         PQclear(res);
	 *term_in << "\\> Trying to delete old database...\n";
         slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
         "DROP DATABASE %s", dbname);

         res = PQexec(check_conn, dbcomm_str);
         if (PQresultStatus(res) != PGRES_COMMAND_OK)
         {
            *term_in << "\\> SQL Command failed...\n\\> " << PQresultErrorMessage(res);
            PQclear(res);
            return (False);
         }
      }
      else
      {
         *term_in << "\\> Database " << dbname << " not found. Creating new one...\n";
         PQclear(res);
      }
   }
   else 
   {
      *term_in << "\\> Can't fetch information about database" << dbname << "\n";
      PQclear(res);
      return(False);
   }

   /* here we should create new database */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "CREATE DATABASE %s", dbname);

   res = PQexec(check_conn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_COMMAND_OK)
   {
      *term_in << "\\> SQL Command failed...\n\\> " << PQresultErrorMessage(res);
      PQclear(res);
      return (False);
   }

   PQfinish(check_conn);
}

int TMyApp::db_prepare(char *dbname, char *dbuser, char *dbpass, 
			char *dbaddr, char *dbport)
{
   PGconn   *check_conn;
   PGresult *res;
   BOOL check_result[TABLES_NUMBER];
   cstring dbcomm_str;

   *term_in << "\\> Checking database " << dbname <<  " integrity...\n";
   /* First mark all tests failed */
   for(int i=0; i<TABLES_NUMBER; i++) check_result[i] = False;

   /* database template1 connect */
   char	    *pgoptions, *pgtty;

   pgoptions = NULL;	/* special options for backend server */
   pgtty     = NULL;	/* debugging tty for the backend server */

   /* check if specified database exist & mark result */
   /* make a connection to the database */
   check_conn = PQsetdbLogin(dbaddr, dbport, pgoptions, pgtty, 
                                "template1", dbuser, dbpass);

   if (PQstatus(check_conn) == CONNECTION_BAD)
   {
      *term_in << "\\> Can't check database... Connection failed.\n";
      *term_in << PQerrorMessage(check_conn);
      exit(-1);
   }
	
   PQsetNoticeProcessor(check_conn, 
   (void (*)(void *, const char *))uNoticeStub, NULL);
    
   /* exec select command on backend server */
   slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
           "SELECT COUNT(*) FROM pg_database WHERE datname like '%s'", dbname);

   res = PQexec(check_conn, dbcomm_str);
   if (PQresultStatus(res) != PGRES_TUPLES_OK)
   {
      *term_in << "\\> SQL Command failed...\n\\> " << PQresultErrorMessage(res);
      PQclear(res);
      return (False);
   }
   
   if (PQntuples(res) > 0) 
   {
      if (atoi(PQgetvalue(res, 0, 0)) > 0) 
      {
         check_result[DB_EXIST] = True;
         PQclear(res);
      }
      else
      {
         *term_in << "\\> Database " << dbname << " not found, please create it with createdb tool\n";
         PQclear(res);
         exit(-1);
      }
   }
   else 
   {
      *term_in << "\\> Can't fetch information about database" << dbname << "\n";
      PQclear(res);
      exit(-1);
   }

   PQfinish(check_conn);
   
   /* now it is time to connect to specified database and check all tables */
   check_conn = PQsetdbLogin(dbaddr, dbport, pgoptions, pgtty, 
                                dbname, dbuser, dbpass);

   if (PQstatus(check_conn) == CONNECTION_BAD)
   {
      *term_in << "\\> Can't connect db to check tables...\n";
      *term_in << "\\> " << PQerrorMessage(check_conn);
      exit(-1);
   }
	
   PQsetNoticeProcessor(check_conn, 
   (void (*)(void *, const char *))uNoticeStub, NULL);
   
   /* check if mess table exist and correct */
   check_result[MESS_TBL_OK] = check_user_tbl(check_conn, 
                               MESS_TBL_NAME, MESS_TBL_FIELDS);
   /* check if lock table exist and correct */
   check_result[LOCK_TBL_OK] = check_user_tbl(check_conn, 
                               LOCK_TBL_NAME, LOCK_TBL_FIELDS);   
   /* check if user table exist and correct */
   check_result[USER_TBL_OK] = check_user_tbl(check_conn, 
                               USER_TBL_NAME, USER_TBL_FIELDS);
   /* check if deps table exist and correct */
   check_result[DEPS_TBL_OK] = check_user_tbl(check_conn, 
                               DEPS_TBL_NAME, DEPS_TBL_FIELDS);
   /* check if cont table exist and correct */
   check_result[CONT_TBL_OK] = check_user_tbl(check_conn, 
                               CONT_TBL_NAME, CONT_TBL_FIELDS);
   /* check if onln table exist and correct */
   check_result[ONLN_TBL_OK] = check_user_tbl(check_conn, 
                               ONLN_TBL_NAME, ONLN_TBL_FIELDS);
   /* check if dfrg table exist and correct */
   check_result[DFRG_TBL_OK] = check_user_tbl(check_conn, 
                               DFRG_TBL_NAME, DFRG_TBL_FIELDS);

   /* delete old corrupted tables and create new one */
   if (check_result[MESS_TBL_OK] != True) 
   {
       *term_in << "\\> Users_messages table missed or corrupted.\n";
       *term_in << "\\> Creating users_messages table...\n";
       delete_user_tbl(check_conn, MESS_TBL_NAME);
       if (check_result[MESS_TBL_OK] = 
           create_user_tbl(check_conn, MESS_TBL_CREATE) != True)
       {
        *term_in << "\\> Users_messages table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, MESS_TBL_NAME, lp_db_user());;
   }

   if (check_result[LOCK_TBL_OK] != True) 
   {
       *term_in << "\\> Users_lock table missed or corrupted.\n";
       *term_in << "\\> Creating users_lock table...\n";
       delete_user_tbl(check_conn, LOCK_TBL_NAME);
       if (check_result[LOCK_TBL_OK] = 
          create_user_tbl(check_conn, LOCK_TBL_CREATE) != True)
       {
        *term_in << "\\> Users_lock table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, LOCK_TBL_NAME, lp_db_user());
   }

   if (check_result[USER_TBL_OK] != True) 
   {
       *term_in << "\\> Users_info table missed or corrupted.\n";
       *term_in << "\\> Creating users_info table...\n";
       delete_user_tbl(check_conn, USER_TBL_NAME);
       if (check_result[USER_TBL_OK] = 
           create_user_tbl(check_conn, USER_TBL_CREATE) != True)
       {
        *term_in << "\\> Users_info table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       create_user_tbl(check_conn, USER_TBL_CREATE1);
       grant_permissions(check_conn, USER_TBL_NAME, lp_db_user());
   }

   if (check_result[DEPS_TBL_OK] != True) 
   {
       *term_in << "\\> Users_deps table missed or corrupted.\n";
       *term_in << "\\> Creating users_deps table...\n";
       delete_user_tbl(check_conn, DEPS_TBL_NAME);
       if (check_result[DEPS_TBL_OK] = 
           create_user_tbl(check_conn, DEPS_TBL_CREATE) != True)
       {
        *term_in << "\\> Users_deps table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, DEPS_TBL_NAME, lp_db_user());
   }

   if (check_result[CONT_TBL_OK] != True) 
   {
       *term_in << "\\> Online_contacts table missed or corrupted.\n";
       *term_in << "\\> Creating online_contacts table...\n";
       delete_user_tbl(check_conn, CONT_TBL_NAME);
       if (check_result[CONT_TBL_OK] = 
           create_user_tbl(check_conn, CONT_TBL_CREATE) != True)
       {
        *term_in << "\\> Online_contacts table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, CONT_TBL_NAME, lp_db_user());
   }

   if (check_result[ONLN_TBL_OK] != True) 
   {
       *term_in << "\\> Online_users table missed or corrupted.\n";
       *term_in << "\\> Creating online_users table...\n";
       delete_user_tbl(check_conn, ONLN_TBL_NAME);
       if (check_result[ONLN_TBL_OK] = 
           create_user_tbl(check_conn, ONLN_TBL_CREATE) != True)
       {
        *term_in << "\\> Online_users table can't be created.\n";
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, ONLN_TBL_NAME, lp_db_user());
   }

   if (check_result[DFRG_TBL_OK] != True) 
   {
       *term_in << "\\> Fragment_storage table missed or corrupted.\n";
       *term_in << "\\> Creating fragment_storage table...\n";
       delete_user_tbl(check_conn, DFRG_TBL_NAME);
       if (check_result[DFRG_TBL_OK] = 
           create_user_tbl(check_conn, DFRG_TBL_CREATE) != True)
       {
        *term_in << "\\> Fragment_storage table can't be created.\n"; 
	*term_in << "\\> Please check login options...\n";
       }
       grant_permissions(check_conn, DFRG_TBL_NAME, lp_db_user());
   }
    
   PQfinish(check_conn);
   
   for (int i=0;i<TABLES_NUMBER;i++) 
       if (check_result == False) 
       { 
         *term_in << "\\> Can't fix database. Please check login options.\n";
         return(False);
       }
       
   *term_in << "\\> Database structure OK... \n\\> Check & fix finished...\n";
   return(True);
}
