/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "icontrol.h"

class TClockView;
class TStatView;

extern pstring debugf; 
extern pstring systemf; 
extern pstring alarmf;
extern pstring usersf;

extern BOOL append_log;

struct TServinfo sinfo;

struct dblogin_data
{
    char ddb_user[64];
    char ddb_pass[64];
    char ddb_addr[64];
    char ddb_port[64];
};

/* global database login options */
struct dblogin_data database_login;

/* global server processes list */
TPCollection *server_plist;
TPListDialog *plistdlg;
otstream     *term_in;
BOOL plist_refresh = True;

TMyApp::TMyApp() :
	TProgInit( 
	   &TMyApp::initStatusLine, 
	   &TMyApp::initMenuBar,
 	   &TMyApp::initDeskTop)
{
 initCommands();

 
 TRect r = getExtent();                      // Create the clock view.
 r.a.x = r.b.x - 9;      r.b.y = r.a.y + 1;
 sclock = new TClockView( r );
 insert(sclock);

 r = getExtent();                            // Create the status view.
 r.a.x = r.a.x + 2; r.a.y = r.b.y - 1;
 sstat = new TStatView( r );
 insert(sstat);

 deskTop->insert(new TEditWindow(TRect(1,1,60,15), ICQ_CONFIG_FILE, 2));

 r = getExtent();
 r.a.x = r.b.x - 61 ; r.b.x--; r.a.y = 2; r.b.y -= 4;
 
 tterm = new TTerm(r); 
 deskTop->insert(tterm);

 sinfo.lbtime = -1;
 term_in = new otstream(tterm->term_view) ;
 
 *term_in << "\\> IControl started... \n";
 *term_in << "\\> Terminal initialized...\n";

 server_plist = new TPCollection(5, 5);
 
 init_bindinterface();
 init_translate();      /* load translation table */
 server_addr = bind_interface;
 
 bind_interface.s_addr = INADDR_ANY;
 udpserver_start(0); sstatus = NONE;
 fcntl(msockfd, F_SETFL, O_NONBLOCK);
 
 send_data_request();
 
 tm_old = time(NULL);
 tm_new = tm_old;
 info_open   = False;
} 

void TMyApp::show_plist()
{
 plistdlg = new TPListDialog( server_plist );
 deskTop->execView(plistdlg);
}

void TMyApp::start_server()
{
 int tmp_pid;
 
 char *pargv[2];
 char *ppath = ICQ_SERVER_FILE;

 pargv[0] = ppath;
 pargv[1] = NULL;
 
 *term_in << "\\> Starting iserverd as daemon... ";
 
 tmp_pid = fork();
 
 if (tmp_pid != 0) {
 if (tmp_pid < 0) {
 
   *term_in << "Failed\n";
   return;
 }
   *term_in << "OK\n";
   return; 
 }
 
 execv(ICQ_SERVER_FILE, pargv);
 exit(0);
}

void TMyApp::send_data_request()
{

 Packet pack;
 pack << (unsigned short)SYS_PROTO
      << (unsigned short)ICQ_SYSxRCV_SYSINFO_REQ
      << (unsigned short)(strlen(lp_info_passwd())+1)
      << lp_info_passwd();
 
 pack.from_ip   = server_addr;
 pack.from_port = lp_udp_port();
 udp_send_direct_packet(pack);

}

void TMyApp::check_data_reply()
{
 Packet pack;
 unsigned short pvers, pcomm;
 unsigned short pcount, prrole;
 unsigned long  prsize, prpid;
 
 /* check for broadcast timeout (only if lbtime != -1) */
 if ((sinfo.lbtime != -1) && (time(NULL) - sinfo.lbtime == 5))
 {
   *term_in << "\\> Reply timeout. Broadcast was not sent...";
   sinfo.lbtime = -1;
 }
 
 /* check for server status */
 if (udp_recv_pack(pack))
 {
   sinfo.lutime = time(NULL);
   pack >> (pvers)
        >> (pcomm);
   if ((pvers == SYS_PROTO) && 
       (pcomm == ICQ_SYSxSND_SYSINFO_REP))
   {
      pack >> (sinfo.bind_port)
           >> (sinfo.bind_addr)
   	   >> (sinfo.server_started)
	   >> (sinfo.config_loaded)
	   >> (sinfo.pack_processed)
	   >> (sinfo.pack_in_queue)
	   >> (sinfo.queue_size)
	   >> (sinfo.used_memory)
	   >> (sinfo.sp_pid)
	   >> (sinfo.ep_pid)
	   >> (sinfo.pp_num)
	   >> (sinfo.bp_num)
	   >> (pcount);

	   if ((plistdlg) && (plist_refresh))
	   {
              server_plist = new TPCollection(5,5);
       
              for (int i=0; i<pcount; i++)
	      {
	         pack >> prpid
	              >> prrole
		      >> prsize;
	      
	         server_plist->insert(new TPEntry(prpid, prrole));
	      }
	   
	      plistdlg->PList->showProcesses( server_plist );
	      plist_refresh = False;
	   }
	   
      if (info_open) 
      {
         i_win->set_info_data(sinfo);
         i_win->update();
      }
      
      sstat->ch_status("Server status: started...");
      if (sstatus != STARTED )
      {
         *term_in << "\\> Server started with pid " << sinfo.sp_pid << "\n";
	 plist_refresh = True;
      }
      enableCommand(cm_serverstop);
      enableCommand(cm_serverrestart);
      enableCommand(cm_cfgreload);
      enableCommand(cm_Broadcast);
      enableCommand(cm_PCoredump);
      enableCommand(cm_PSigkill);

      disableCommand(cm_serverstart);
      sstatus = STARTED;

   }
   else
   {
      if ((pvers == SYS_PROTO) &&
          (pcomm == ICQ_SYSxSND_PASSWD_ERR))
      {
         sstat->ch_status("Server status: Online (but info password error)...");
	 if (sstatus != PASSWORD)
	     *term_in << "\\> Info password is incorrect (or default), but server started\n";
	 disableCommand(cm_serverstart);
	 disableCommand(cm_Broadcast);
	 disableCommand(cm_serverstop);
	 disableCommand(cm_serverrestart);
	 disableCommand(cm_cfgreload);
	 sstatus = PASSWORD;
      }
      
      if ((pvers == SYS_PROTO) &&
          (pcomm == ICQ_SYSxSND_SYSBCAST_REP))
      {
	 *term_in << "\\> Broadcast message sent successfully...";
	 sinfo.lbtime = -1;
      }
   }
 } 
 else
 {
   if ((time(NULL)-sinfo.lutime) > 5)
   {
      disableCommand(cm_serverstop);
      disableCommand(cm_serverrestart);
      disableCommand(cm_cfgreload);
      
      disableCommand(cm_Broadcast);
      disableCommand(cm_PCoredump);
      disableCommand(cm_PSigkill);
      
      enableCommand(cm_serverstart);
      
      sstat->ch_status("Server status: stopped...");
      if (sstatus != STOPPED)
      {
          *term_in << "\\> Info timeout, server stopped...\n";
          sstatus = STOPPED;
          if ((plistdlg))
          {
             server_plist = new TPCollection(5,5);
             plistdlg->PList->showProcesses( server_plist );
             plist_refresh = False;
          }
      }
   }
 }
}

static Boolean isInfoWin(TView *p, void * )
{
   if( (p->options & ofWinInfo) != 0)
       return True;
   else
       return False;
}

void TMyApp::idle()
{
    
    TProgram::idle();
    tm_old = tm_new;
    tm_new = time(NULL);
    
    if( (tterm) && ((tterm->state & sfVisible) != 0) )
    {
       disableCommand(cm_ShowTerm);
    }
    else
    {
       enableCommand(cm_ShowTerm);
    }
    
    if (tm_new > tm_old)
    {
      sclock->update();

      
      if (deskTop->firstThat(isInfoWin, 0) == 0 ) 
         { info_open = False; enableCommand(cm_ShowInfo); }

         send_data_request();
         check_data_reply();
	 
      if (info_open) 
      {

         i_win->update();
      }
    }
}

TStatusLine *TMyApp::initStatusLine(TRect r)
{
    r.a.y = r.b.y - 1;     // move top to 1 line above bottom
    return new TStatusLine( r,
        *new TStatusDef( 0, 0xFFFF ) +
        // set range of help contexts
            *new TStatusItem( 0, kbF10, cmMenu )+
	    *new TStatusItem( 0, kbCtrlL, cm_Refresh)+
            *new TStatusItem( 0, kbAltX, cmQuit )

        );
}

void TMyApp::initCommands()
{
    disableCommand(cm_serverstart);
    disableCommand(cm_serverstop);
    disableCommand(cm_cfgreload);
    disableCommand(cm_serverrestart);
    disableCommand(cm_None);
}


void TMyApp::run() { execute();}

void TMyApp::handleEvent(TEvent& event)
{
    TApplication::handleEvent(event);
    if( event.what == evCommand )
        {
        switch( event.message.command )
            {
            case cm_serverstart:
		start_server();
		break;
            case cm_serverstop:
	        *term_in << "\\> Sent KILL signal to pid " << sinfo.sp_pid << "\n";
		kill(sinfo.sp_pid, SIGINT);
		break;
	    case cm_serverrestart:
		*term_in << "\\> Sent KILL signal to pid " << sinfo.sp_pid << "\n";
	        kill(sinfo.sp_pid, SIGALRM);
		start_server();
	        break;
	    case cm_cfgreload:
		*term_in << "\\> Sent HUP signal to pid " << sinfo.sp_pid << "\n";
		kill(sinfo.sp_pid, SIGHUP);
		break;
	    case cm_dblogin:
	        dblogin_dialog();
		break;
	    case cm_Refresh:
		win_refresh();
		break;
	    case cm_Prepare:
	        db_prepare(lp_db_users(),
    			    database_login.ddb_user,
			    database_login.ddb_pass,
			    database_login.ddb_addr,
			    database_login.ddb_port);
		break;
	    case cm_Purge:
		 if (db_recreate(lp_db_users(),
		             database_login.ddb_user,
			     database_login.ddb_pass,
			     database_login.ddb_addr,
			     database_login.ddb_port) != False)
		 {
	                    db_prepare(lp_db_users(),
    			    database_login.ddb_user,
			    database_login.ddb_pass,
			    database_login.ddb_addr,
			    database_login.ddb_port);
		 }
		 break;
	    case cm_EditConfig:
                 deskTop->insert(new TEditWindow(TRect(1,1,60,15), ICQ_CONFIG_FILE, 2));
		 break;
		 		 
	    case cm_ShowInfo:
	        show_info_window();
		info_open = True;
		disableCommand(cm_ShowInfo);
		break;
	    
	    case cm_ShowPlist:
		show_plist();
		break;
		
	    case cm_ShowAbout:
		ShowAbout();
		break;	
	
	    case cm_Suspend:
		raise(SIGTSTP);
		break;
		
	    case cm_ShowTerm:
		tterm->show();
		break;
	    
	    case cm_Broadcast:
		admin_bcast();
		break;
		
            default:
                return;
            }
        clearEvent( event );
        }
}

void TMyApp::show_info_window()
{
  struct TServinfo sinfo;
  TRect r = getExtent();
  
  r.a.x = 2; r.b.x = 40; r.a.y = 2; r.b.y = 19;
  
//  i_win = new TWinInfo(TRect( 40, 3, 78, 20), "Server Info..." );
  i_win = new TWinInfo(r, "Server Info..." );
  deskTop -> insert(i_win);
  
  sinfo.lutime = time(NULL)-10;
  
  i_win -> set_info_data(sinfo);
  i_win -> update();
}

void TMyApp::dblogin_dialog()
{
    int Control;
    
    TDialog *pd = new TDialog( TRect( 20, 6, 60, 16), "Connect database" );
    if( pd )
    {
	class TView *b, *b1;

	b1 = new TInputLine( TRect( 13, 2, 38, 3), 64);
        pd->insert( b1 ); pd->insert( new TLabel( TRect( 1, 2, 12, 3), "Username:", b1 ));

	b = new TInputPass( TRect( 13, 4, 38, 5), 64);
        pd->insert( b ); pd->insert( new TLabel( TRect( 1, 4, 12, 5), "Password:", b ));

	b = new TInputLine( TRect( 13, 6, 28, 7), 64);
        pd->insert( b ); pd->insert( new TLabel( TRect( 1, 6, 12, 7), "Server:", b ));

	b = new TInputLine( TRect( 31, 6, 38, 7), 64);
        pd->insert( b ); pd->insert( new TLabel( TRect( 28, 6, 30, 7), ":", b ));

        pd->insert( new TButton( TRect( 12, 8, 24, 9 ), "~O~K", cmOK, bfDefault ));
        pd->insert( new TButton( TRect( 26, 8, 38, 9 ), "~C~ancel", cmCancel, bfNormal ));
        
	pd->setData( &database_login );
	b1->focus();
	
	Control = deskTop->execView( pd );
	if (Control != cmCancel) pd->getData( &database_login );
        
    }
    destroy( pd );
}


/* This just init & run tvision application */
int main(int argc, char **argv)
{
  switch_to_server_root(argv[0]);

 /* Process command line options */
  init_globals();
  pstring configf;
  slprintf(configf, sizeof(configf)-1, lp_config_file());

  /* Global parameters initialization */

  slprintf(debugf,  sizeof(debugf),  lp_dbglog_path());
  umask(022); append_log = lp_append_logs(); reopen_logs(); 

  /* Loading configuration file */
  if (!lp_load(configf,False,False,True))
  {
    printf("FATAL ERROR: Can't find config file: \"%s\"\n", configf);
    exit(0);
  }

  strncpy(database_login.ddb_user, lp_db_user(), 63);
  strncpy(database_login.ddb_pass, lp_db_pass(), 63);
  strncpy(database_login.ddb_port, lp_db_port(), 63);
  strncpy(database_login.ddb_addr, lp_db_addr(), 63);

  TMyApp myApp;
  myApp.run();
  return 0;
}
