/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#define Uses_TMemo
#define Uses_TCollection
#define Uses_TListBox
#define Uses_otstream
#define Uses_TTerminal
#define Uses_TEditor
#define Uses_TEditWindow
#define Uses_TCommandSet
#define Uses_TEventQueue
#define Uses_TEvent
#define Uses_TProgram
#define Uses_TApplication
#define Uses_TKeys
#define Uses_TRect
#define Uses_TMenuBar
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TStatusLine
#define Uses_TStatusItem
#define Uses_TStatusDef
#define Uses_TDeskTop
#define Uses_TView
#define Uses_TWindow
#define Uses_TFrame
#define Uses_TScroller
#define Uses_TScrollBar
#define Uses_TDialog
#define Uses_TButton
#define Uses_TSItem
#define Uses_TCheckBoxes
#define Uses_TRadioButtons
#define Uses_TLabel
#define Uses_TInputLine
#define Uses_TInputPass
#define Uses_MsgBox

#include <tvision/tv.h>
#include "includes.h"

#ifndef __SERVINFO
#define __SERVINFO

typedef struct TServinfo
{
  unsigned long  lutime;
  unsigned long  bind_addr;
  unsigned long  bind_port;
  unsigned long  pack_processed;
  unsigned long  pack_in_queue;
  unsigned long  queue_size;
  unsigned long  used_memory;
  unsigned long  config_loaded;
  unsigned long	 server_started;
  unsigned long  sp_pid;
  unsigned long  ep_pid;
  unsigned long  pp_num;
  unsigned long  bp_num;
  long  lbtime;

} TServinfo;

#endif

#include "TWinInfo.h"
#include "TClockView.h"
#include "TTerminal.h"
#include "TPListDialog.h"
#include "TEditInput.h"

#define STARTED  1
#define STOPPED  2
#define PASSWORD 3
#define NONE	 4

#define TABLES_NUMBER 8

class TMyApp : public TApplication
{

public:
	TMyApp();
	
        static TStatusLine *initStatusLine( TRect r );
        static TMenuBar *initMenuBar( TRect r );
	virtual void handleEvent( TEvent& event);
	static void initCommands();
	virtual void run();
	virtual void idle();
	void dblogin_dialog();
	void show_info_window();
	void send_data_request();
	void check_data_reply();
	void start_server();
	void show_plist();
	void ShowAbout();
	void admin_bcast();
	BOOL  db_recreate(char *dbname, char *dbuser, char *dbpass, 
			 char *dbaddr, char *dbport);
	BOOL  db_prepare(char *dbname, char *dbuser, char *dbpass, 
			char *dbaddr, char *dbport);

private:
        TClockView  *sclock;
	TTerm       *tterm;
	TStatView   *sstat;
	TEditWindow *e_win;
	TWinInfo    *i_win;
	int	    sstatus;
	BOOL	    info_open;
	time_t	    tm_old, tm_new;
	struct in_addr server_addr;
};

const int
     cm_PRefresh	= 120,
     cm_PCoredump	= 121,
     cm_PSigkill	= 122;

const int
     cm_None		= 100,
     cm_serverstart	= 101,
     cm_serverstop	= 102,
     cm_serverrestart	= 103,
     cm_Prepare		= 104,
     cm_dblogin		= 105,
     cm_Refresh		= 106,
     cm_getstatus	= 107,
     cm_ShowInfo	= 108,
     cm_Purge		= 109,
     cm_cfgreload	= 110,
     cm_ShowPlist	= 111,
     cm_Suspend		= 112,
     cm_ShowTerm	= 113,
     cm_ShowAbout	= 114,
     cm_Broadcast	= 115,
     cm_EditConfig	= 116;
     
