/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*                                                                        */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*                                                                        */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#ifndef _INCLUDES_H
#define _INCLUDES_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h> 
#include <stddef.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/resource.h>
#include <sys/param.h>
#include <sys/ioctl.h>

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <net/if.h>

#include <sys/un.h>
#include <poll.h>

#include <syslog.h> 

#ifdef TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#if defined(HAVE_SIGACTION) && defined(HAVE_SIGPROCMASK) && defined(HAVE_SIGADDSET)
# include <signal.h>
#endif

#include "defaults.h"
#include "talloc.h"
#include "dlinklist.h"

#ifndef _BOOL
#define _BOOL
#define False (0)
#define True (1)
#define Auto (2)
typedef int BOOL;
#endif

#ifndef _SMODE
#define _SMODE
#define MOD_STANDALONE (0)
#define MOD_DAEMON (1)
#define MOD_INETD  (2)
typedef int SMODE;
#endif

#ifndef _PSTRING                  
#define PSTRING_LEN 1024
#define FSTRING_LEN 256
#define CSTRING_LEN 1024
#define MSTRING_LEN 2048
typedef char bigstring[MAX_PACKET_SIZE+1024];
typedef char pstring[PSTRING_LEN];
typedef char fstring[FSTRING_LEN];
typedef char cstring[CSTRING_LEN];
typedef char mstring[MSTRING_LEN];
#define _PSTRING
#endif                            

#define uint32 unsigned int

#ifndef uchar
#define uchar unsigned char
#endif

#define LIST_SEP " \t,;:\n\r"
enum case_handling {CASE_LOWER,CASE_UPPER};

#include "safe_string.h"
#include "macros.h"
#include "debug.h"
#include "log_sys.h"
#include "log_alr.h"
#include "log_usr.h"
#include "lock.h"
#include "process.h"
#include "packet.h"
#include "database/database.h"
#include "database/init_db.h"
#include "event/confirm.h"
#include "event/online.h"
#include "message/sys_proto/vsys_defines.h"

#ifdef HAVE_POSTGRESDB
#include "libpq-fe.h"
#include "libpq-fs.h"
#endif

#include "public.h"
#include "proto.h"
#include "proto_defines.h"
#include "interfaces.h"
#include "translate.h"
#include "protos.h"

#ifndef O_ACCMODE                               
#define O_ACCMODE (O_RDONLY | O_WRONLY | O_RDWR)
#endif                                          

#define SLAB(LEN, POS)	LEN - POS >= 4 ? 4 : LEN - POS
#define ZERO_STRUCTPN(x) memset((char *)(x), 0, sizeof(*(x)))
#define ip_equal(ip1,ip2) ((ip1).s_addr == (ip2).s_addr)
#define putip(dest,src) memcpy(dest,src,4)

#ifndef HAVE_MEMCPY
#define	memcpy(s1, s2, n)	bcopy(s2, s1, n)
#endif

#define OPTSTR  		"oVd:c:p:"

#ifdef _SEM_SEMUN_UNDEFINED
union semun {
    int val;                           /* <= value for SETVAL */
    struct semid_ds *buf;              /* <= buffer for IPC_STAT & IPC_SET */
    unsigned short int *array;         /* <= array for GETALL & SETALL */
    struct seminfo *__buf;             /* <= buffer for IPC_INFO */
};
#endif

#ifndef __IPC_STRUCTURE
#define __IPC_STRUCTURE
typedef struct ipc_structure
{
  unsigned long  pack_in_queue;	        /* number of packets in pipe */
  unsigned short magic_num1;		/* random number1 for system injector */
  unsigned short magic_num2;		/* random number2 for system injector */

} ipc_structure;

#endif

#ifndef HAVE_SOCKLEN_T
#ifdef RECVFROM_WITH_INT
#define socklen_t int
#else
typedef unsigned int unint_def;
#define socklen_t unint_def
#endif
#endif

#ifndef __CHILD_LIST
#define __CHILD_LIST
/* child record for childs list */
struct child_process
{
    pid_t child_pid;		 // child pid number
    int   child_role;		 // child function
    
};

/* childs list element record */
typedef struct child_record
{
    struct child_record  *prior; 
    struct child_record  *next;
    struct child_process child;
    
} child_record;
#endif

#endif /* INCLUDES_H */
