/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"

// return short (16-bit) stored in little-endian format
unsigned short get_le_short(char *p)
{
   unsigned char *q = (unsigned char *)p;
   return q[0] + (q[1] << 8);
}

// return int (32-bit) stored in little-endian format
unsigned int get_le_int(char *p)
{
   unsigned char *q = (unsigned char *)p;
   return q[0] + (q[1] << 8) + (q[2] << 16) + (q[3] << 24);
}

// return long (32-bit) stored in little-endian format
unsigned long get_le_long(char *p)
{
   unsigned char *q = (unsigned char *)p;
   return q[0] + (q[1] << 8) + (q[2] << 16) + (q[3] << 24);
}

// store 16-bit short in little-endian format
void put_le_short(unsigned short x, char *p)
{
   unsigned char *q = (unsigned char*)p;
   q[0] = x & 0xff;
   q[1] = (x >> 8) & 0xff;
}

// store 32-bit int in little-endian format
void put_le_int(unsigned int x, char *p)
{
   unsigned char *q = (unsigned char*)p;
   q[0] = x & 0xff;
   q[1] = (x >> 8) & 0xff;
   q[2] = (x >> 16) & 0xff;
   q[3] = (x >> 24) & 0xff;
}

// store 32-bit int in little-endian format
void put_le_long(unsigned long x, char *p)
{
   unsigned char *q = (unsigned char*)p;
   q[0] = x & 0xff;
   q[1] = (x >> 8) & 0xff;
   q[2] = (x >> 16) & 0xff;
   q[3] = (x >> 24) & 0xff;
}

Packet::Packet(void)
{
   maxSize = MAX_PACKET_SIZE;
//   buff = new char[maxSize];
   nextData = buff;
   sizeVal = 0;
}

// packet class constructor, i've removed new() operators because of
// incorrect work on some computers :((( 
Packet::Packet(Packet *packet)
{
   maxSize = packet->maxSize;
//   buff = new char[maxSize];
   nextData = packet->nextData;
   sizeVal = packet->sizeVal;   
   for (long i = 0; i < sizeVal; i++) buff[i] = packet->buff[i];
}


// packet class constructor, i've removed new() operators because of
// incorrect work on some computers :((( 
Packet::Packet(char *newBuff, unsigned long buffSize)
{
    maxSize = MAX_PACKET_SIZE;
    if (buffSize > maxSize) buffSize = maxSize;
//    buff = new char[maxSize];
    memcpy(buff, newBuff, buffSize);
    nextData = buff;
    sizeVal = buffSize;
}

// packet destructor
Packet::~Packet()
{
//    delete(buff);
}

char *Packet::print(void)
{
    static char p[2048];
    char *pPos = p;

    slprintf(pPos, sizeof(pPos)-1, "  ");
    pPos += 2;
    for(long i = 0; i < sizeVal; i++)
    {
       slprintf(pPos, sizeof(pPos)-1, "%02X ", (unsigned char)buff[i]);
       pPos += 3;
       if((i + 1) % 24 == 0) { slprintf(pPos, sizeof(pPos)-1, "\n  "); pPos += 3; }
    }
    slprintf(pPos, sizeof(pPos)-1, "\n%c", '\0');
    return(p);
}

void Packet::clearPacket(void)
{
   nextData = buff;
   sizeVal = 0;
}


Packet& Packet::operator<<(unsigned int data)
{
   put_le_int(data, nextData);
   sizeVal += sizeof(unsigned int);
   nextData += sizeof(unsigned int);
   return(*this);    
}


Packet& Packet::operator<<(unsigned short data)
{
   put_le_short(data, nextData);
   sizeVal += sizeof(unsigned short);
   nextData += sizeof(unsigned short);
   return(*this);    
}


Packet& Packet::operator<<(unsigned long data)
{
   put_le_long(data, nextData);
   sizeVal += sizeof(unsigned long);
   nextData += sizeof(unsigned long);
   return(*this);    
}


Packet& Packet::operator<<(char data)
{
   int s = sizeof(unsigned char);
   memcpy(nextData, &data, s);
   sizeVal += s;
   nextData += s;
   return(*this);    
}


Packet& Packet::operator<<(char *data)
{
   int s = strlen((char *)data) + 1;
   memcpy(nextData, data, s);
   sizeVal += s;
   nextData += s;
   return(*this);
}

Packet& Packet::operator>>(char &in)
{
   if(nextData + sizeof(char) > (buff + sizeVal)) 
      in = 0;
   else
   {
      in = *((char *)nextData);
      nextData += sizeof(char);
   }
   return(*this);
}


Packet& Packet::operator>>(unsigned short &in)
{
   if(nextData + sizeof(unsigned short) > (buff + sizeVal)) 
      in = 0;
   else
   {
      in = get_le_short(nextData);
      nextData += sizeof(unsigned short);
   }
   return(*this);
}


Packet& Packet::operator>>(unsigned long &in)
{
   if(nextData + sizeof(unsigned long) > (buff + sizeVal)) 
      in = 0;
   else
   {
      in = get_le_long(nextData);
      nextData += sizeof(unsigned long);
   }
   return(*this);
}

Packet& Packet::operator>>(long &in)
{
   if(nextData + sizeof(long) > (buff + sizeVal)) 
      in = 0;
   else
   {
      in = get_le_long(nextData);
      nextData += sizeof(long);
   }
   return(*this);
}

Packet& Packet::operator>>(short &in)
{
   if(nextData + sizeof(short) > (buff + sizeVal)) 
      in = 0;
   else
   {
      in = get_le_short(nextData);
      nextData += sizeof(short);
   }
   return(*this);
}
