/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"
#include "v3_proto/v3_defines.h"

/**************************************************************************/
/* Determine who we are and do specific work 				  */
/**************************************************************************/
void process_server()
{ 
  while(1)					  // repeat forever
  {
    switch (process_role)
    {
      case ROLE_SOCKET: process_socket(); break;  // we are main parent
      case ROLE_PACKET: process_packet(); break;  // we are packet-processor 
      case ROLE_EVENT : process_event();  break;  // we are event-processor
      case ROLE_BUSY  : process_busy();   break;  // we are busy_processor
      case ROLE_NONE  : exit(0);	  break;  // if we haven't role
    }
  }
}

/**************************************************************************/
/* func, working with main server socket. It receive			  */
/* all packets in blocking mode and write them to			  */
/* special nonamed pipe.						  */
/**************************************************************************/
void process_socket()
{
  unsigned short pversion;	/* to check if packet -> system packet    */
  pack_processed = 0;		/* number of processed by server packets  */
  Packet upacket;		/* udp socket processor temporal packet   */
  Packet wpacket;		/* unix socket wwp temporal packet	  */
  
  /* we should create fd matrix for poll function - MS and WS */
  sock_fds[SUDP].fd = msockfd;
  sock_fds[SUDP].events  = POLLIN;
  sock_fds[SUDP].revents = 0;

  sock_fds[SWWP].fd = wwpsockfd;
  sock_fds[SWWP].events  = POLLIN;
  sock_fds[SWWP].revents = 0;

  LOG_SYS(10, ("Init: Ready to serve requests on sockets.\n"));

  while(1)
  {
    /* block on sockets array for infinity time */
    if (poll(sock_fds, 2, -1) < 1)
    {
      if (errno != EINTR) 
      {
         LOG_SYS(10, ("We have problem with poll(): %s\n", strerror(errno)));
      }
    }
    
    /* check if we have incoming client udp message */
    if (isready_data(SUDP)) 
    {
       udp_receive_packet(upacket);
       upacket.reset(); upacket >> pversion;

       if (pversion == SYS_PROTO)
       {
          handle_sys_proto(upacket);
       }
       else
       {
          /* sanity check - to avoid wwp flooding */
          if (ipToIcq(upacket.from_ip) != 0)
	  {
             pipe_send_packet(upacket);
             ipc_vars->pack_in_queue++;
             pack_processed++;
	  }
       }
    }
    
    /* check if we have incoming wwp message */
    if (isready_data(SWWP)) 
    {
        wwp_receive_packet(wpacket);
        handle_wwp_mess(wpacket);
    }

    /* check if we have lost any child */
    if (childs_check == 1)
    {
      /* check & fork new if need */
      check_packet_processors(); 	
      if (process_role != ROLE_SOCKET) return;
      check_event_processor();
      if (process_role != ROLE_SOCKET) return;
    }
  }
}

/**************************************************************************/
/* func, working with unnamed pipe. it set dec 				  */
/* semaphore, read from pipe and inc semaphore. Then it 		  */
/* process packet. All ack packets it send directly to 			  */
/* socket, all confirmed packet it send to second 			  */
/* pipe - to event processor						  */
/**************************************************************************/
void process_packet()
{
  Packet ppacket;

  init_all_protocols();
  usersdb_connect();			/* connect to users database */
  
  for(;;)
  {
    pipe_receive_packet(ppacket);	/* receive packet from pipe */
    handle_icq_packet(ppacket);		/* handle packet */
    ipc_vars->pack_in_queue--;
  }
  
  return;
}

/**************************************************************************/
/* func, working with second unnamed pipe and timer. It			  */
/* receive from pipe packets, send them to socket,			  */
/* insert them in cycle event list and when ack arrived			  */
/* delete it from list							  */
/**************************************************************************/
void process_event()
{
  indirect_pack *ppacket = new indirect_pack;
  struct event_pack epacket;
 
  LOG_SYS(10, ("Init: Event processor initialization success.\n"));
  
  /* signal initialization */
  rsignal(SIGALRM,  (void(*)(int))eventSIGALRMHandler);
   
  init_packets_list();		/* initialization of confirm (ask) list */
  usersdb_connect();		/* connect to database server 		*/
  init_online_cache();		/* get online users from db & validate 	*/
  ualarm(1000000, 1000000);   	/* we need alarm after each 1 sec 	*/
       
  /* we should create fd matrix for poll function - OS and ES 	*/
  evnt_fds[SOUT].fd 	 = outgoing_pipe_fd;
  evnt_fds[SOUT].events  = POLLIN;
  evnt_fds[SOUT].revents = 0;

  evnt_fds[SEVT].fd 	 = event_pipe_fd;
  evnt_fds[SEVT].events  = POLLIN;
  evnt_fds[SEVT].revents = 0;

  /* now main work cycle for event-processor*/
  while(1)
  {
     /* block on sockets array for infinity time */
     if (poll(evnt_fds, 2, -1) < 1)
     {
       if (errno != EINTR) 
       {
          LOG_SYS(10, ("We have problem with poll(): %s\n", strerror(errno)));
	  exit(-1);
       }
     }
 
     /* check if we have outgoing packet in pipe  */
     if (isready_data2(SOUT)) 
     {
	event_receive_packet(*ppacket);	/* receive packet from pipe 	*/
	event_process_packet(*ppacket);	/* send it & add to list 	*/
     }


     /* check if we have event notification in pipe */
     if (isready_data2(SEVT))
     {
         event_receive_event(epacket);	/* receive event from pipe	*/
         event_process_event(epacket);	/* parse event record	 	*/
     }
  }
}

/**************************************************************************/
/* func, working with unnamed pipe. It receive packets 			  */
/* and send to socket "busy" message. It used to avoid pipe		  */
/* overflow on heavy server load (overload)				  */
/* Warning: live-time of this processor = 60 sec			  */
/**************************************************************************/
void process_busy()
{
 DEBUG(0,("PROCESSING BUSY\n"));
 exit(-2);
}

/**************************************************************************/
/* Childs list  WARNING: don't use after list creation. It can 		  */
/* cause a memory leak				 			  */
/**************************************************************************/
void init_childs_list()
{
 cl_begin = NULL;
 cl_list =  NULL;
 cl_last =  NULL;
}

/**************************************************************************/
/* Insert new child to list 						  */
/**************************************************************************/
void child_insert(pid_t child_pid, int role)
{
  cl_last = (child_record *)malloc(sizeof(*cl_last));
  
  DEBUG(100,("Insert child: %d (role: %d)\n",child_pid, role));
  
  cl_last->child.child_pid  = child_pid;
  cl_last->child.child_role = role;
  
  if (cl_begin == NULL) 		// list is empty
  {
    cl_last->prior = NULL;
    cl_begin = cl_last;
    cl_last->next = NULL;
  }
  else					// insert before first 
  {
    cl_list = cl_begin;
    cl_begin = cl_last;
    cl_begin->next = cl_list;
    cl_begin->next->prior = cl_begin;
    cl_begin->prior = NULL;
    
  }
}

/**************************************************************************/
/* Delete child with specified pid from childs list			  */
/**************************************************************************/
void child_delete(pid_t child_pid)
{
  struct child_record * ltemp = NULL;
  cl_list = cl_begin;

  while(cl_list)
  {
    if (cl_list->child.child_pid == child_pid)
    {
      // delete from middle of list
      if ((cl_list->prior != NULL) & (cl_list->next != NULL))
      {
        cl_list->prior->next = cl_list->next;	  // relink pointers
	cl_list->next->prior = cl_list->prior;
	ltemp = cl_list;
	cl_list = cl_list->next;
	free(ltemp);			  	  // release memory
	return;
      }
      
      // list contain only one record
      if ((cl_list->prior == NULL) & (cl_list->next == NULL))
      {
        cl_begin = NULL;
	free(cl_list);
	return;
      }      
      
      // delete record from end of the list
      if (cl_list->next == NULL)
      {
        cl_list->prior->next = NULL;
	free(cl_list);
	return;
      }
      // delete record from begin of the list
      if (cl_list->prior == NULL)
      { 
        cl_begin = cl_list->next;
	cl_list->next->prior = NULL;
	free(cl_list);
	return;
      }
    }
    cl_list = cl_list->next;
  }    
}

/**************************************************************************/
/* Search child with specified pid and return it's role 		  */
/**************************************************************************/
int child_lookup(pid_t child_pid)
{
 cl_list = cl_begin;
 while(cl_list)
 {
   if (cl_list->child.child_pid == child_pid) 
           return (cl_list->child.child_role);
   cl_list = cl_list->next;
 }
 return(-1);
}

/**************************************************************************/
/* Return number of childs with determined role. If role= -1 		  */
/* it return number of all childs 					  */
/**************************************************************************/
int childs_num(int role)
{
  int child_num = 0;
  cl_list = cl_begin;
  
  while(cl_list)
  {
    if (role == -1)			// return total number of childs
    {
      child_num += 1;
    }
    else
    {
      if (cl_list->child.child_role == role)
      {
        child_num += 1;
      }
    }
    cl_list = cl_list->next;
  }
  return(child_num);
}


/**************************************************************************/
/* Return pid of event-processor 					  */
/**************************************************************************/
int geteppid()
{
  int child_pid = 0;
  cl_list = cl_begin;
  
  while(cl_list)
  {
    if (cl_list->child.child_role == ROLE_EVENT)
    {
        child_pid = cl_list->child.child_pid;
	break;
    }
    cl_list = cl_list->next;
  }
  return(child_pid);
}

/**************************************************************************/
/* Check number of packet processors and fork new if need		  */
/**************************************************************************/
void check_packet_processors()
{
 if (childs_num(ROLE_PACKET) < lp_min_childs())
 {
   fork_packet_processors(lp_min_childs()-childs_num(ROLE_PACKET));
   childs_check = 0;
 }
}

/**************************************************************************/
/* Check if a event processor is alive and fork 			  */
/* new if it is dead							  */
/**************************************************************************/
void check_event_processor()
{
 if (childs_num(ROLE_EVENT) == 0)
 {
   fork_event_processor();
   childs_check = 0;
 }
}

/**************************************************************************/
/* Check if specified socket have unread data 	 			  */
/**************************************************************************/
BOOL isready_data(int number)
{
  if ((sock_fds[number].revents & POLLIN) != 0) return(True);
  return(False);
}

/**************************************************************************/
/* Check if specified socket have unread data 	 			  */
/**************************************************************************/
BOOL isready_data2(int number)
{
  if ((evnt_fds[number].revents & POLLIN) != 0) return(True);
  return(False);
}

/**************************************************************************/
/* Server cleanup. Close files, kill childs, remove pidfile, etc 	  */
/**************************************************************************/
void Server_cleanup()
{
  union semun ss;
  struct shmid_ds sm;
  memset(&sm, 0, sizeof(sm));
  memset(&ss, 0, sizeof(ss));
 
  /* do it only if we are parent */
  if (process_role == ROLE_SOCKET)
  {
    /* remove lock file */
    Unlock(pidFP); fclose(pidFP); unlink(lp_pid_path());

    /* close main udp socket */
    close(msockfd);
    /* close interprocess pipes */
    close(incoming_pipe_fd);
    close(outgoing_pipe_fd);
    close(event_pipe_fd);

    /* kill all childs - send SIGINT to process group */
    no_deth_messages = True;
    signal(SIGINT, SIG_IGN);   /* we don't need this signal here */
    kill(0, SIGINT);

    /* remove IPC objects (semaphores, msg queues, shared mem, etc) */
    shmdt((char *)ipc_vars);
    semctl(semaphore_id, 0, IPC_RMID, ss);
    msgctl(messqueue_id, 0, IPC_RMID);
    shmctl(sharedmem_id, IPC_RMID, &sm);

    /* remove unix domain sockets (used as IPC pipes) */
    if (unlink(ip_serv_addr.sun_path) < 0)
    {
      LOG_SYS(0, ("CLEANUP: Can't remove [%s]\n", ip_serv_addr.sun_path));
      LOG_SYS(0, ("Error: [%s]\n", strerror(errno)));
    }
    
    if (unlink(op_serv_addr.sun_path) < 0)
    {
      LOG_SYS(0, ("CLEANUP: Can't remove [%s]\n", op_serv_addr.sun_path));
      LOG_SYS(0, ("Error: [%s]\n", strerror(errno)));
    }

    if (unlink(wp_serv_addr.sun_path) < 0)
    {
      LOG_SYS(0, ("CLEANUP: Can't remove [%s]\n", wp_serv_addr.sun_path));
      LOG_SYS(0, ("Error: [%s]\n", strerror(errno)));
    }

    if (unlink(ep_serv_addr.sun_path) < 0)
    {
      LOG_SYS(0, ("CLEANUP: Can't remove [%s]\n", ep_serv_addr.sun_path));
      LOG_SYS(0, ("Error: [%s]\n", strerror(errno)));
    }


    LOG_SYS(0, ("EXIT: Cleanup finished. Bye...\n"));
  }
}

/**************************************************************************/
/* Handle webpager messages from wwp-socket 			 	  */
/**************************************************************************/
void handle_wwp_mess(Packet &pack)
{
   unsigned  long signature;
   unsigned  long tuin;
   unsigned  long from_ip;
   unsigned short str_len;
   int i;
   
   char email[64];
   char name[32];
   char mess[450];
   char mess1[550];
   char mess2[550];
   pack.reset();
   
   /* extract data from packet */
   pack >> signature >> from_ip >> tuin >> str_len;
   
   if (signature != 0x01020709) 
   {
      DEBUG(100, ("Wrong signature in wwp packet...\n"));  
   }
   
   if (str_len > sizeof(email)+1) 
   {
      DEBUG(10, ("WWP Email string len overflow...\n"));
      return;
   }
   for (i=0; i<str_len; i++) pack >> email[i];

   pack >> str_len;
   if (str_len > sizeof(name)+1)
   {
      DEBUG(10, ("WWP Name string len overflow...\n"));
      return;
   }

   for (i=0; i<str_len; i++) pack >> name[i];

   pack >> str_len;
   if (str_len > sizeof(mess)+1)
   {
      DEBUG(10, ("WWP Message string len overflow...\n"));
      return;
   }

   for (i=0; i<str_len; i++) pack >> mess[i];
   
   snprintf(mess1, sizeof(mess1)+1, 
            "This message was sent from: %s\x0d\x0a--------------------------------------------------------------------------\x0d\x0a", 
	    inet_ntoa(icqToIp(from_ip)));

   snprintf(mess2, sizeof(mess2)-1, "%s%s", mess1, mess);
   
   /* now we should generate non-standart v3-packet 
      and send it to packet-processor thru pipe */
   reply_pack.clearPacket();
   reply_pack << (unsigned short)V3_PROTO
              << (unsigned short)ICQ_CMDxRCV_WWP_MSG
              << (unsigned short)0x0000
	      << (unsigned short)0x0000
	      << (unsigned  long)0x00000001
	      << (unsigned  long)0x00000000
	      << (unsigned  long)tuin
	      << (unsigned short)(strlen(name)+1)
	      << name
	      << (unsigned short)(strlen(email)+1)
	      << email
	      << (unsigned short)(strlen(mess2)+1)
	      << mess2;
	            
   pipe_send_packet(reply_pack); 
}
