/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"

/* semaphore control data structures */
/* this is used to lock outgoing pipe for io */
static struct sembuf out_lock[2]    = { {1, 0, 0}, {1, 1, 0} };
static struct sembuf out_unlock[1]  = { {1,-1, 0} };
/* this is used to lock incoming pipe for io */
static struct sembuf inc_lock[2]    = { {0, 0, 0}, {0, 1, 0} };
static struct sembuf inc_unlock[1]  = { {0,-1, 0} };
/* this is used to lock log files for io */
static struct sembuf wp_lock[2]     = { {2, 0, 0}, {2, 1, 0} };
static struct sembuf wp_unlock[1]   = { {2,-1, 0} };

static struct sembuf incw_lock[2]   = { {3, 0, 0}, {3, 1, 0} };
static struct sembuf incw_unlock[1] = { {3,-1, 0} };

/*************************************************************************/
/* Create semaphore and increase it to 1. 1 - pipe free, 0 - pipe locked */
/*************************************************************************/
void ipc_objects_init()
{
  semun ss;
  memset(&ss, 0, sizeof(ss));
  
  semaphore_id = semget(IPC_PRIVATE, 4, 0777|IPC_CREAT|IPC_EXCL);
  if (semaphore_id == -1)
  {
    LOG_SYS(0, ("Can't create semaphores. Error: [%s]\n", strerror(errno)));
    exit(-1);
  }

  semctl(semaphore_id, 0, SETVAL, ss);
  semctl(semaphore_id, 1, SETVAL, ss);
  semctl(semaphore_id, 2, SETVAL, ss);
  semctl(semaphore_id, 3, SETVAL, ss);

  inc_lock[0].sem_flg    = SEM_UNDO;  inc_lock[1].sem_flg   = SEM_UNDO;
  inc_unlock[0].sem_flg  = SEM_UNDO;  inc_unlock[1].sem_flg = SEM_UNDO;
  
  out_lock[0].sem_flg    = SEM_UNDO;  out_lock[1].sem_flg   = SEM_UNDO;
  out_unlock[0].sem_flg  = SEM_UNDO;  out_unlock[1].sem_flg = SEM_UNDO;

  wp_lock[0].sem_flg     = SEM_UNDO;  wp_lock[1].sem_flg   = SEM_UNDO;
  wp_unlock[0].sem_flg   = SEM_UNDO;  wp_unlock[1].sem_flg = SEM_UNDO;

  incw_lock[0].sem_flg   = SEM_UNDO;  incw_lock[1].sem_flg   = SEM_UNDO;
  incw_unlock[0].sem_flg = SEM_UNDO;  incw_unlock[1].sem_flg = SEM_UNDO;

  sems_ready = 1;
  
  /* now we should init message queue */
  if (( messqueue_id = msgget(IPC_PRIVATE, 0777 | IPC_CREAT )) < 0 ) 
  {
    LOG_SYS(0, ("ERROR: Can't create IPC messages queue. Exiting...\n"));
    exit(-1);
  }
  
  /* shared memory structure initialization */
  sharedmem_id = shmget(IPC_PRIVATE, sizeof(ipc_structure), SHM_R | SHM_W);
  if (sharedmem_id == -1)
  {
    LOG_SYS(0, ("Can't allocate shared memory structure. 
		 Error: [%s]\n", strerror(errno)));
    exit(-1);
  }

  ipc_vars = (ipc_structure *)shmat(sharedmem_id, 0, 0);
  if (sharedmem_id == -1)
  {
    LOG_SYS(0, ("Can't attach shared memory segment. 
		 Error: [%s]\n", strerror(errno)));
    exit(-1);
  }
  
  ipc_vars->pack_in_queue = 0;
  ipc_vars->magic_num1 = random_num();
  ipc_vars->magic_num2 = random_num();
  
}

/**************************************************************************/
/* Lock interprocess pipe for reading/writing 				  */
/**************************************************************************/
void lock_pipe(int pipe)
{
 // waiting for unlock (when sem = 0)
 if (pipe == INCOMING_PIPE) semop(semaphore_id, &inc_lock[0], 2);
 if (pipe == OUTGOING_PIPE) semop(semaphore_id, &out_lock[0], 2); 
}

/**************************************************************************/
/* Unlock interprocess pipe for reading/writing 			  */
/**************************************************************************/
void unlock_pipe(int pipe)
{
  if (pipe == INCOMING_PIPE) semop(semaphore_id, &inc_unlock[0], 1);
  if (pipe == OUTGOING_PIPE) semop(semaphore_id, &out_unlock[0], 1);
}

/**************************************************************************/
/* Lock incoming pipe for writing		 			  */
/**************************************************************************/
BOOL lock_wpipe()
{
 // waiting for unlock (when sem = 0) and lock
 semop(semaphore_id, &wp_lock[0], 2);
 
 return(True); 
}

/**************************************************************************/
/* Unlock incoming pipe for writing			 		  */
/**************************************************************************/
BOOL unlock_wpipe()
{
  semop(semaphore_id, &wp_unlock[0], 1);

  return(True);
}

/**************************************************************************/
/* Lock incoming pipe for write 	 				  */
/**************************************************************************/
BOOL lock_incw()
{
 // waiting for unlock (when sem = 0) and lock
 semop(semaphore_id, &incw_lock[0], 2);
 
 return(True);
}

/**************************************************************************/
/* Unlock incoming pipe after writing to it				  */
/**************************************************************************/
BOOL unlock_incw()
{
  semop(semaphore_id, &incw_unlock[0], 1);
  
  return(True);
}


/**************************************************************************/
/* Set internal process_role variable to PACKET_ROLE, fork		  */
/* another process and add it to child_list 				  */
/**************************************************************************/
void fork_packet_processors(int num)
{
  int temp;
  pid_t child_pid;
  int i;
  
  /* we need to fork specified number of childs */
  for  (i=0; i<num; i++)
  {
    temp = process_role;
    process_role = ROLE_PACKET;
    child_pid = fork();
    if (child_pid != 0)
    {
     /* We are parent process, we need return back process_role 
        and push pid to the list of child processes. */  
        process_role = temp;
        child_insert(child_pid, ROLE_PACKET);
    }
    else 
    {
        childSignals_Init();
        return;
    }
  }
}

/**************************************************************************/
/* Set internal process_role variable to EVENT_ROLE, fork another 	  */
/* process and add it to child_list				  	  */
/**************************************************************************/
void fork_event_processor()
{
  int temp;
  pid_t child_pid;
  
  temp = process_role;
  process_role = ROLE_EVENT;
  child_pid = fork();
  if (child_pid != 0)
  {
   /* We are parent process, we need return back process_role 
      and push pid to the list of child processes. */
      process_role = temp;
      child_insert(child_pid, ROLE_EVENT);  // it is packet-processor child
      return;
  }
  else
  {
      childSignals_Init();
      return;
  }
}


/**************************************************************************/
/* Fork several packet-processors and one event processor	  	  */
/**************************************************************************/
void fork_childs()
{
  childs_check = 0;
  process_role = ROLE_SOCKET;

  init_childs_list();
  fork_packet_processors(lp_min_childs());

  /* fork one process working with event pipe and timer */
  if (process_role == ROLE_SOCKET) fork_event_processor();
}

/**************************************************************************/
/* Switch to server directory.                             	  	  */
/**************************************************************************/
void switch_to_server_root(char *bin_path)
{

#ifdef ENABLE_SWITCH
  
  int slen;
  char npath[128];
  
  /* we need to remove filename from command string */
  for (int i=strlen(bin_path); i>=0; i--)
     if (bin_path[i] == '/') { slen = i+1; break; }
      
  /* sanity check */
  if (slen > 128) slen = 128;
  snprintf(npath, slen+1, bin_path);
  
  /* chdir to server_root/bin */
  if (chdir(npath) != 0)
  {
    printf("Can't switch to server root: \"%s\"\n", strerror(errno));
    exit(-1);
  }
  
  /* chdir to server_root (../) */
  if (chdir("../") != 0)
  {
    printf("Can't switch to server root: \"%s\"\n", strerror(errno));
    exit(-1);
  }
    
  getcwd(npath, 128);
  
#else

  chdir("/");

#endif /* ENABLE_SWITCH */

}


/**************************************************************************/
/* Translate table initialization.                             	  	  */
/**************************************************************************/
void init_translate()
{
   char temp[128];
   
   snprintf(temp, 128, "%s/%s", lp_translate_file(), lp_translate_tbl());
   ITrans.setTranslationMap(temp);
   
}

