/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"
#include "v3_defines.h"

extern int v3_timeout; 
extern int v3_retries;

/**************************************************************************/
/* System message packet handler 				    	  */
/**************************************************************************/
void v3_process_sysmsg()
{
 unsigned short pvers, pcomm, seq1, seq2, msg_type, msg_len, i;
 unsigned long  uin_num, to_uin, temp_stamp;
 struct online_user user, to_user;
 mstring msg_buff;

 int_pack.reset();
 int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num >> temp_stamp
          >> to_uin >> msg_type >> msg_len;
 
 if (db_online_lookup(uin_num, user) == 0)
 {
    if (ipToIcq(user.ip) == ipToIcq(int_pack.from_ip))
    {
      v3_send_ack(int_pack);
      int_pack.reset();
      int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num >> temp_stamp
               >> to_uin >> msg_type >> msg_len;
      
      /* extract new password from packet */
      if (!islen_valid(msg_len, lp_v3_max_msgsize(), "message", user))
      {
  	 move_user_offline(user.uin);
	 v3_send_not_connected(int_pack);
	 return;
	 
      } else { for(i=0; i<msg_len; i++) int_pack >> msg_buff[i]; }
      
      if (db_online_lookup(to_uin, to_user) == 0)
      {
	DEBUG(100, ("ONMessage (%d), \"%s\"\n", msg_type, msg_buff));
	send_online_message(user, to_user, msg_type, seq2, msg_buff);
      } else {
      	DEBUG(100, ("OFMessage (%d), \"%s\"\n", msg_type, msg_buff));
        db_add_message(uin_num, to_uin, msg_type, msg_buff);
      }
    } else {
       LOG_ALARM(0, ("Spoofed msg thru server command from %s:%d (%lu)\n", 
       inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num));
    }   
 } else {

   v3_send_not_connected(int_pack);

 }
}


/**************************************************************************/
/* Broadcast message packet handler 				    	  */
/**************************************************************************/
void v3_process_broadcast()
{
 unsigned short pvers, pcomm, seq1, seq2, msg_type, msg_len, i;
 unsigned long  uin_num, to_uin, temp_stamp, department;
 struct online_user user;
 struct login_user_info user_info;
 mstring msg_buff;
 
 log_alarm_packet(100, int_pack);

 int_pack.reset();
 int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num >> temp_stamp
          >> to_uin >> msg_type >> msg_len;
 
 if (db_online_lookup(uin_num, user, seq1, seq2) == 0)
 {
    if (ipToIcq(user.ip) == ipToIcq(int_pack.from_ip))
    {
      if ((uin_num != ADMIN_UIN) && (db_users_lookup(uin_num, user_info) != 0)) return;
      if ((uin_num != ADMIN_UIN) && (user_info.can_broadcast != 1)) 
      {
	LOG_ALARM(0, ("User %lu sent broadcast but haven't rights...\n", 
		       uin_num));
	return;
      }
    
      v3_send_ack(int_pack);
      
      int_pack.reset();
      int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num >> temp_stamp
               >> department >> msg_type >> msg_len;
      
      if (!islen_valid(msg_len, lp_v3_max_msgsize(), "message", user))
      {
  	 move_user_offline(user.uin);
	 v3_send_not_connected(int_pack);
	 return;
	 
      } else { for(i=0; i<msg_len; i++) int_pack >> msg_buff[i]; }
          
      if (pcomm == ICQ_CMDxRCV_BROADCAST_MSG_ALL)
      {
         broadcast_message(False, department, msg_buff, 0x14, uin_num);
      } 
      else
      {
         broadcast_message(True, department, msg_buff, 0x14, uin_num);
      }
      
    } else {
       LOG_ALARM(0, ("Spoofed broadcast message command from %s:%d (%lu)\n", 
       inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num));
    }   
 } else {
   v3_send_not_connected(int_pack);
 }
}

/**************************************************************************/
/* Send message to user with V3 proto  				    	  */
/**************************************************************************/
int v3_send_user_message(unsigned long from_user_uin, 
			  struct online_user &to_user, 
			  int type, unsigned short seq2, char *message)
{

  reply_pack.clearPacket();
  reply_pack << (unsigned short)V3_PROTO
             << (unsigned short)ICQ_CMDxSND_SYSxMSGxONLINE
             << (unsigned short)to_user.servseq++
             << (unsigned short)seq2
             << (unsigned  long)to_user.uin
             << (unsigned  long)0x0000
	     << (unsigned  long)from_user_uin
	     << (unsigned short)type
	     << (unsigned short)(strlen(message)+1)
	     << message;
	     
   DEBUG(10, ("Info: Sending online system message to %lu\n", to_user.uin));

   reply_pack.from_ip   = to_user.ip;
   reply_pack.from_port = to_user.udp_port;
   
   /* sending packets to client, we need confirm! */
   return(v3_send_indirect(reply_pack, to_user.uin, to_user.servseq-1));
}


/**************************************************************************/
/* Process system message done ack  				    	  */
/**************************************************************************/
void v3_process_sysack()
{
 unsigned short pvers, pcomm, seq1, seq2;
 unsigned long  uin_num, temp_stamp;
 struct online_user user;

 int_pack.reset();
 int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num;
 
 if (db_online_lookup(uin_num, user) == 0)
 {
    if (ipToIcq(user.ip) == ipToIcq(int_pack.from_ip))
    {  
      v3_send_ack(int_pack);
    
      int_pack.reset();
      int_pack >> pvers >> pcomm >> seq1 >> seq2 
               >> uin_num >> temp_stamp >> temp_stamp;

      db_del_messages(uin_num, temp_stamp);
      
    } else {
    
       LOG_ALARM(0, ("Spoofed set password command from %s:%d (%lu)\n", 
       inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num));
    }   
 } else {
 
   v3_send_not_connected(int_pack);

 }
}

/**************************************************************************/
/* Send offline message to user via V3 proto  			    	  */
/**************************************************************************/
int v3_send_user_sysmsg(unsigned long from_uin, 
			  struct online_user &to_user, 
			  int type, unsigned short seq2, 
			  unsigned long mess_time, char *message)
{
  int part_cnt = 0;
  reply_pack.clearPacket();
  reply_pack << (unsigned short)V3_PROTO
             << (unsigned short)ICQ_CMDxSND_SYSxMSGxOFFLINE
             << (unsigned short)to_user.servseq
             << (unsigned short)seq2
             << (unsigned  long)to_user.uin
             << (unsigned  long)0x0000
	     << (unsigned  long)from_uin
	     << (unsigned  long)mess_time
	     << (unsigned short)type
	     << (unsigned short)(strlen(message)+1)
	     << message;
	     
   DEBUG(10, ("Info: Sending offline system message to %lu\n", to_user.uin));

   reply_pack.from_ip   = to_user.ip;
   reply_pack.from_port = to_user.udp_port;
   
   /* sending packets to client, we need confirm! */
   part_cnt = v3_send_indirect(reply_pack, to_user.uin, to_user.servseq);
   to_user.servseq += part_cnt;
   
   return(part_cnt);
   
}

/**************************************************************************/
/* Process system and offline messages request		    		  */
/**************************************************************************/
void v3_process_sysmsg_req()
{
 unsigned short pvers, pcomm, seq1, seq2, mess_num, pack_num;
 unsigned long  uin_num, dep_number, temp_stamp;
 struct online_user user;
 PGresult *res;
 cstring dbcomm_str; 
 char **valid = NULL;

 int_pack.reset();
 int_pack >> pvers >> pcomm >> seq1 >> seq2 >> uin_num 
          >> temp_stamp >> temp_stamp >> dep_number;
 
 if (db_online_lookup(uin_num, user) == 0)
 {
    if (ipToIcq(user.ip) == ipToIcq(int_pack.from_ip))
    {
      v3_send_ack(int_pack);
  
      db_online_sseq_open(user); pack_num = 0;
      
      slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
            "DECLARE mportal CURSOR FOR SELECT * FROM Users_Messages 
	     WHERE to_uin=%d ORDER BY msg_date", uin_num);

      res = PQexec(users_dbconn, dbcomm_str);
      if (PQresultStatus(res) != PGRES_COMMAND_OK)
      {
         LOG_SYS(0, ("SQL Command failed...\n"));
         LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
         PQclear(res);
         return;
	 
      }  else PQclear(res);
      
      bigstring message;
      unsigned long from_uin, msg_date;
      unsigned short msg_type;
            
      for (mess_num=0;;mess_num++)
      {
         res = PQexec(users_dbconn, "FETCH IN mportal");
         if (PQresultStatus(res) != PGRES_TUPLES_OK)
         {
            LOG_SYS(0, ("SQL Command failed...\n"));
            LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
	    PQclear(res);
            break;
         }
	 
	 if (PQntuples(res) == 0) {PQclear(res); break;}

	 strncpy(message, PQgetvalue(res, 0, 4), sizeof(message)-1);
	 ITrans.translateToClient(message);
	 
	 from_uin  = strtoul(PQgetvalue(res, 0, 1), valid, 10); valid = NULL;
	 msg_date  = strtoul(PQgetvalue(res, 0, 2), valid, 10); valid = NULL;
	 msg_type  = atol(PQgetvalue(res, 0, 3)); 
	 pack_num += v3_send_user_sysmsg(from_uin, user, msg_type, 
	                                 mess_num, msg_date, message);
	 
	 PQclear(res);
      }
      
      PQclear(PQexec(users_dbconn, "CLOSE mportal"));
      v3_send_reply_ok(int_pack, user, ICQ_CMDxSND_SYSxMSGxDONE);
      db_online_sseq_close(user, pack_num+1);
      return;
      
    } else {
    
       LOG_ALARM(0, ("Spoofed sys messages request from %s:%d (%lu)\n", 
       inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num));
    }   
 } else {
 
   v3_send_not_connected(int_pack);
 }
}

