/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"
#include "v5_defines.h"

/**************************************************************************/
/* User change status packet					   	  */
/**************************************************************************/
void v5_send_user_status(struct online_user &to_user, struct online_user &user)
{
   reply_pack.clearPacket();
   
   reply_pack << (unsigned short)V5_PROTO
               << (char)0x00
               << (unsigned  long)to_user.session_id
	       << (unsigned short)ICQ_CMDxSND_USERxSTATUS
               << (unsigned short)to_user.servseq++
               << (unsigned short)0x0000
               << (unsigned  long)to_user.uin
	       << (unsigned  long)0x0000
	       << (unsigned  long)user.uin
	       << (unsigned  long)user.status;

   PutKey(reply_pack, calculate_checkcode(reply_pack));
   
   DEBUG(100, ("Online: Sending alert about %lu to user %lu\n", 
                user.uin, to_user.uin));

   reply_pack.from_ip   = to_user.ip;
   reply_pack.from_port = to_user.udp_port;
   
   /* sending packets to client, we need confirm! */
   v5_send_indirect(reply_pack, to_user.uin, to_user.servseq-1);

}


/**************************************************************************/
/* User online packet						   	  */
/**************************************************************************/
void v5_send_user_online(struct online_user &to_user, struct online_user &user)
{
   reply_pack.clearPacket();
   
   reply_pack << (unsigned short)V5_PROTO
               << (char)0x00
               << (unsigned  long)to_user.session_id
	       << (unsigned short)ICQ_CMDxSND_USERxONLINE
               << (unsigned short)to_user.servseq++
               << (unsigned short)0x0000
               << (unsigned  long)to_user.uin
	       << (unsigned  long)0x0000
	       << (unsigned  long)user.uin
	       << (unsigned  long)ipToIcq(user.ip)
	       << (unsigned  long)user.tcp_port
	       << (unsigned  long)ipToIcq(user.ip) << (char)0x04
	       << (unsigned  long)user.status
	       << (unsigned  long)user.tcpver
	       << (unsigned  long)0x38F6FD33
	       << (unsigned  long)0x50000000
	       << (unsigned  long)0x03000000
	       << (unsigned  long)0xd8cb2a39
	       << (unsigned  long)0x33799939
	       << (unsigned  long)0x34799939;

   PutKey(reply_pack, calculate_checkcode(reply_pack));
   
   DEBUG(100, ("Online: Sending alert about %lu to user %lu\n", 
                user.uin, to_user.uin));

   reply_pack.from_ip   = to_user.ip;
   reply_pack.from_port = to_user.udp_port;
   
   /* sending packets to client, we need confirm! */
   v5_send_indirect(reply_pack, to_user.uin, to_user.servseq-1);
}

/**************************************************************************/
/* User offline packet						   	  */
/**************************************************************************/
void v5_send_user_offline(struct online_user &to_user, unsigned long uin)
{
   reply_pack.clearPacket();
   
   reply_pack << (unsigned short)V5_PROTO
               << (char)0x00  /* gap byte */
               << (unsigned  long)to_user.session_id
	       << (unsigned short)ICQ_CMDxSND_USERxOFFLINE
               << (unsigned short)to_user.servseq++
               << (unsigned short)0x0000
               << (unsigned  long)to_user.uin
	       << (unsigned  long)0x0000
	       << (unsigned  long)uin;

   PutKey(reply_pack, calculate_checkcode(reply_pack));
   
   DEBUG(100, ("Online: Sending alert about %lu to user %lu\n", 
                uin, to_user.uin));

   reply_pack.from_ip   = to_user.ip;
   reply_pack.from_port = to_user.udp_port;
   
   /* sending packets to client, we need confirm! */
   v5_send_indirect(reply_pack, to_user.uin, to_user.servseq-1);
   log_alarm_packet(10, reply_pack);
}


/**************************************************************************/
/* Server should confirm *every* received packet (except client ack). 	  */
/**************************************************************************/
void v5_send_ack(Packet &pack)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
      
 reply_pack.clearPacket();
 
 reply_pack << (unsigned short)V5_PROTO		// proto number
             << (char)0x00			// gap?
             << (unsigned  long)session_id  	// session_id number
	     << (unsigned short)ICQ_CMDxSND_ACK // ack command
             << (unsigned short)seq1		// sequence 1 num
             << (unsigned short)seq2		// sequence 2 num
             << (unsigned  long)uin_num		// client uin number
	     << (unsigned  long)0x0000;
	    
 PutKey(reply_pack, calculate_checkcode(reply_pack));
  
 /* we do not need to encrypt packet */
 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;

 /* sending packets to client */
 udp_send_direct_packet(reply_pack);
 
}

/**************************************************************************/
/* This packet sent if client can't be logged in for some reason   	  */
/**************************************************************************/
void v5_send_login_err(Packet &pack, char *errmessage)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
 
 reply_pack.clearPacket();

 reply_pack << (unsigned short)V5_PROTO		// proto number
             << (char)0x00			// gap?
             << (unsigned  long)session_id  	// session_id number
	     << (unsigned short)ICQ_CMDxSND_LOGIN_ERR
	     << (unsigned short)0x0000		// sequence 1 num
             << (unsigned short)seq2		// sequence 2 num
             << (unsigned  long)uin_num		// client uin number
	     << (unsigned  long)0x0000
	     << (unsigned short)(strlen(errmessage)+1)
	     << errmessage
	     << (unsigned  long)0x0000;

 PutKey(reply_pack, calculate_checkcode(reply_pack));

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;

 /* sending packets to client */
 udp_send_direct_packet(reply_pack);
}

/**************************************************************************/
/* This packet sent if client can't be logged in for some reason   	  */
/**************************************************************************/
void v5_send_test_err(Packet &pack, unsigned short commd, char *errmessage)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
 
 reply_pack.clearPacket();

 reply_pack << (unsigned short)V5_PROTO		// proto number
             << (char)0x00			// gap?
             << (unsigned  long)session_id  	// session_id number
	     << (unsigned short)commd
	     << (unsigned short)0x0000		// sequence 1 num
             << (unsigned short)seq2		// sequence 2 num
             << (unsigned  long)uin_num		// client uin number
	     << (unsigned  long)0x0000
	     << (unsigned short)(strlen(errmessage)+1)
	     << errmessage;

 PutKey(reply_pack, calculate_checkcode(reply_pack));

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;

 /* sending packets to client */
 udp_send_direct_packet(reply_pack);
}

/**************************************************************************/
/* This packet sent if client's password wrong or it doesn't exist 	  */ 
/* in user database 						   	  */
/**************************************************************************/
void v5_send_pass_err(Packet &pack)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
 
 reply_pack.clearPacket();
 reply_pack << (unsigned short)V3_PROTO
	     << (unsigned short)ICQ_CMDxSND_WRONGxPASSWD
             << (unsigned short)0x0000
             << (unsigned short)seq2
             << (unsigned  long)uin_num
             << (unsigned  long)0x0000
	     << (unsigned short)0x0000;

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;

 /* sending packets to client */
 udp_send_direct_packet(reply_pack);

}

/**************************************************************************/
/* Send busy packet to user					   	  */
/**************************************************************************/
void v5_send_busy(Packet &pack)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
      
 reply_pack.clearPacket();
 
 reply_pack << (unsigned short)V5_PROTO	 	 // proto number
             << (char)0x00			 // gap?
             << (unsigned  long)session_id  	 // session_id number
	     << (unsigned short)ICQ_CMDxSND_BUSY // ack command
             << (unsigned short)0x0000		 // sequence 1 num
             << (unsigned short)seq2		 // sequence 2 num
             << (unsigned  long)uin_num		 // client uin number
	     << (unsigned  long)0x0000;
	    
 PutKey(reply_pack, calculate_checkcode(reply_pack));
  
 /* we do not need to encrypt packet */

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;
 
 /* sending packets to client */
 udp_send_direct_packet(reply_pack);

}


/**************************************************************************/
/* Login reply packet						   	  */
/**************************************************************************/
void v5_send_login_reply(Packet &pack, struct login_user_info &userinfo)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2;
 
 reply_pack.clearPacket();

 reply_pack << (unsigned short)V5_PROTO		   // proto number
             << (char)0x00			   // gap?
             << (unsigned  long)session_id  	   // session_id number
	     << (unsigned short)ICQ_CMDxSND_HELLO
	     << (unsigned short)0x0000		   // sequence 1 num
             << (unsigned short)seq2		   // sequence 2 num
             << (unsigned  long)uin_num		   // client uin number
	     << (unsigned  long)0x0000
	     
	     << (unsigned  long)0x8c000000	   // unknown
	     << (unsigned short)0xf000		   // unknown
	     << (unsigned short)0x0a00		   // timeout
	     << (unsigned short)0x0a00		   // unknown
	     << (unsigned short)0x0500		   // retries
	     << (unsigned  long)ipToIcq(reply_pack.from_ip)
	     << (unsigned  long)0x80A3039F;	   // server ID
	     
 PutKey(reply_pack, calculate_checkcode(reply_pack));

 /* we do not need to encrypt packet */

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;
 
 /* sending packets to client */
 udp_send_direct_packet(reply_pack);

}

/**************************************************************************/
/* Non-logged client tried to work with server 			   	  */
/**************************************************************************/
void v5_send_not_connected(Packet &pack)
{
 unsigned short seq1, seq2, vvers, pcomm;
 unsigned long  uin_num, junk, session_id;

 /* we need seq1, seq2 numbers and uin to */
 pack.reset();
 pack >> vvers >> junk >> uin_num >> session_id 
      >> pcomm >> seq1 >> seq2 ;
      
 reply_pack.clearPacket();
 
 reply_pack << (unsigned short)V5_PROTO
             << (char)0x00		
             << (unsigned  long)session_id
	     << (unsigned short)ICQ_CMDxSND_ERR_NOT_CONNECTED
             << (unsigned short)0x0000
             << (unsigned short)seq2
             << (unsigned  long)uin_num
	     << (unsigned  long)0x0000;
	    
 PutKey(reply_pack, calculate_checkcode(reply_pack));
  
 /* we do not need to encrypt packet */

 reply_pack.from_ip   = pack.from_ip;
 reply_pack.from_port = pack.from_port;
 
 /* sending packets to client */
 udp_send_direct_packet(reply_pack);

}

/**************************************************************************/
/* Send busy packet to user					   	  */
/**************************************************************************/
void v5_send_end_contact(struct online_user &user)
{
 reply_pack.clearPacket();
 
 reply_pack << (unsigned short)V5_PROTO
             << (char)0x00
             << (unsigned  long)user.session_id
	     << (unsigned short)ICQ_CMDxSND_USERxLISTxDONE
             << (unsigned short)user.servseq++
             << (unsigned short)0x0000
             << (unsigned  long)user.uin
	     << (unsigned  long)0x0000;
	    
 PutKey(reply_pack, calculate_checkcode(reply_pack));
  
 /* we do not need to encrypt packet */
 reply_pack.from_ip   = user.ip;
 reply_pack.from_port = user.udp_port;
 
 /* sending packets to client */
 v5_send_indirect(reply_pack, user.uin, user.servseq-1);

}

