/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"
#include "v5_defines.h"

int v5_timeout; int v5_retries;

/**************************************************************************/
/* Main v5 packet handlers selector 					  */
/* If packet can't be handled, selector write it to alarm log lev=(200)	  */
/**************************************************************************/
void handle_v5_proto(Packet &pack)
{
 unsigned short pvers, pcomm;
 unsigned long  uin_num, ljunk, session_id;
 
 int_pack = pack;
  
 V5Decrypt(int_pack);	/* unpack packet and check its checkcode */
 int_pack.reset(); 	/* set data pointer to packet begin 	 */
 
 /* we need some data from packet :) */
 int_pack >> pvers   >> ljunk 
	   >> uin_num >> session_id 
	   >> pcomm;

 /* Well now we need to determine what we got... */
 switch (pcomm)
 {
    case ICQ_CMDxRCV_LOGON:  		v5_process_login(); 	  break;
    case ICQ_CMDxRCV_ACK:		v5_process_ack();         break;
    case ICQ_CMDxRCV_USERxLIST:		v5_process_contact();     break;
    case ICQ_CMDxRCV_LOGOFF:		v5_process_logoff();      break;
    case ICQ_CMDxRCV_PING:		v5_process_ping();        break;
    case ICQ_CMDxRCV_PING2:		v5_process_ping();        break;
    case ICQ_CMDxRCV_LOGINMETA:		v5_process_lmeta();	  break;
    case ICQ_CMDxRCV_SETxSTATUS:	v5_process_status();      break;    
    
    default:

         log_alarm_packet(0, int_pack);  /* dump unknown packet */
	 v5_send_ack(int_pack); break;	 /* and ask it 	 */
 }
}


/**************************************************************************/
/* Status change hanlder						  */
/**************************************************************************/
void v5_process_status()
{
  unsigned short pvers, pcomm, seq1, seq2;
  unsigned long  uin_num, temp_stamp, session_id, new_status;
  struct online_user user;

  int_pack.reset();
  int_pack >> pvers   >> temp_stamp
	    >> uin_num >> session_id
	    >> pcomm   >> seq1
            >> seq2    >> temp_stamp
	    >> new_status;

  if (db_online_lookup(uin_num, user) == 0)
  {
    if ((ipToIcq(user.ip) == ipToIcq(int_pack.from_ip)) &&
	(user.session_id == session_id))
    {
      v5_send_ack(int_pack);
      
      DEBUG(10, ("Process user status change command from %s:%d (%lu)\n", 
      inet_ntoa(int_pack.from_ip), int_pack.from_port, user.uin));

      if (new_status == ICQ_STATUS_PRIVATE)
      {
         user.status = new_status;
         db_online_setstatus(user);
	 broadcast_offline( user.uin );
      
      }
	
      if ((new_status != ICQ_STATUS_PRIVATE) & (user.status == ICQ_STATUS_PRIVATE))
      {
      	 user.status = new_status;
         db_online_setstatus(user);
	 broadcast_online( user );

      }
      
      if ((new_status  != ICQ_STATUS_PRIVATE) & 
          (user.status != ICQ_STATUS_PRIVATE))
      {
	 user.status = new_status;
         db_online_setstatus(user);
	 broadcast_status( user );
      }
      
    } else {

    LOG_ALARM(0, ("Spoofed (%lu) status change message from %s:%d\n", uin_num,
		  inet_ntoa(int_pack.from_ip), int_pack.from_port));
    }
  } else {

    v5_send_not_connected(int_pack);
    
  }
}



/**************************************************************************/
/* Unknown packet 							  */
/**************************************************************************/
void v5_process_lmeta()
{
  unsigned short pvers, pcomm, seq1, seq2;
  unsigned long  uin_num, temp_stamp, session_id, meta_data;
  struct online_user user;

  int_pack.reset();
  int_pack >> pvers   >> temp_stamp
	    >> uin_num >> session_id
	    >> pcomm   >> seq1
            >> seq2    >> temp_stamp 
	    >> meta_data;

  if (db_online_lookup(uin_num, user) == 0)
  {
    if ((ipToIcq(user.ip) == ipToIcq(int_pack.from_ip)) &&
	(user.session_id == session_id))
    {
    
      v5_send_ack(int_pack);
      
      DEBUG(10, ("Process user meta command from %s:%d (%lu)\n", 
      inet_ntoa(int_pack.from_ip), int_pack.from_port, meta_data));

      db_online_sseq_open(user);	
      reply_pack.clearPacket();
      reply_pack << (unsigned short)V5_PROTO
                  << (char)0x00
                  << (unsigned  long)user.session_id
	          << (unsigned short)ICQ_CMDxSND_USER_LMETA
                  << (unsigned short)user.servseq++
                  << (unsigned short)0x0000
                  << (unsigned  long)user.uin
	          << (unsigned  long)0x0000
	          << (unsigned  long)meta_data;
		  
      PutKey(reply_pack, calculate_checkcode(reply_pack));
   
      reply_pack.from_ip   = user.ip;
      reply_pack.from_port = user.udp_port;
   
      /* sending packets to client, we need confirm! */
      v5_send_indirect(reply_pack, user.uin, user.servseq-1);
      db_online_sseq_close(user, 1);


    } else {

    LOG_ALARM(0, ("Spoofed (%lu) ping message from %s:%d\n", uin_num,
		  inet_ntoa(int_pack.from_ip), int_pack.from_port));
    }
  } else {
  
    v5_send_not_connected(int_pack);

  }
}


/**************************************************************************/
/* Ping packet hanlder							  */
/**************************************************************************/
void v5_process_ping()
{
  unsigned short pvers, pcomm, seq1, seq2;
  unsigned long  uin_num, temp_stamp, session_id;
  struct online_user user;

  int_pack.reset();
  int_pack >> pvers   >> temp_stamp
	    >> uin_num >> session_id
	    >> pcomm   >> seq1
            >> seq2    >> temp_stamp;

  if (db_online_lookup(uin_num, user) == 0)
  {
    if ((ipToIcq(user.ip) == ipToIcq(int_pack.from_ip)) &&
	(user.session_id == session_id))
    {
    
      v5_send_ack(int_pack);
      
      DEBUG(100, ("Process user ping command from %s:%d (%lu)\n", 
      inet_ntoa(int_pack.from_ip), int_pack.from_port, user.uin));

      LOG_SYS(1, ("Process user ping command from %s:%d (%lu)\n", 
      inet_ntoa(int_pack.from_ip), int_pack.from_port, user.uin));
	
      db_online_touch(user);
      send_update2cache(user.uin, user.ttl);

    } else {

    LOG_ALARM(0, ("Spoofed (%lu) ping message from %s:%d\n", uin_num,
		  inet_ntoa(int_pack.from_ip), int_pack.from_port));
    }
  } else {
  
    v5_send_not_connected(int_pack);

  }
}


/**************************************************************************/
/* Login packet handler							  */
/**************************************************************************/
void v5_process_login()
{
  unsigned short vers, comm, seq1, seq2, i, tcpversion, plen;
  unsigned long uin_num, ljunk, tport, status,  session_id;
  char cjunk;
  struct online_user nuser, check_user;
  struct login_user_info userinfo;
  char usrpass[20];
  fstring lock_text;
  log_alarm_packet(0, int_pack);
  v5_send_ack(int_pack);
  
  int_pack.reset();
  int_pack  >> vers    >> ljunk
	    >> uin_num >> session_id
	    >> comm    >> seq1
            >> seq2    >> ljunk
	    >> ljunk   >> tport;
	   
  int_pack >> plen; if (plen > 20) plen = 20; 
  for (i=0; i<plen; i++) int_pack >> usrpass[i];

  int_pack >> ljunk   >> ljunk
            >> cjunk   >> status >> tcpversion;
	   
  if (!lp_v5_enabled()) 
  {
     LOG_USR(0, ("User %lu try login via V5 proto 
                  (V5 proto disabled)\n", uin_num));
     v5_send_login_err(int_pack, "V5 protocol disabled by administrator...
                       \x0d\x0aTry to use another client version...");
     return;
  }
  
  if (db_users_lookup(uin_num, userinfo) == -1)
  {
    LOG_USR(0, ("Can't find user %lu in database.\n", uin_num));
    v5_send_pass_err(int_pack);
    return;
  }
  
  if (!strcsequal(usrpass, userinfo.passwd))
  {
    LOG_USR(0, ("User %lu trying login with wrong password.\n", uin_num));
    v5_send_pass_err(int_pack);
    return;
  }

  if (userinfo.disabled == 1)
  {
    LOG_USR(0, ("User %lu trying login but his account locked.\n", uin_num));
    if (db_users_lock_message(uin_num, lock_text))
    {  
       ITrans.translateToClient(lock_text);
       v5_send_login_err(int_pack, lock_text);
    } else v5_send_login_err(int_pack, "Account locked...");
    
    return;
  }    
  
  if (db_online_lookup(uin_num, check_user) != 0)
  {
     nuser.uin        = uin_num;
     nuser.ip         = int_pack.from_ip;
     nuser.tcp_port   = tport;
     nuser.udp_port   = int_pack.from_port;
     nuser.status     = status;
     nuser.uptime     = longToTime(time(NULL));
     nuser.lutime     = longToTime(time(NULL));
     nuser.ttl	      = get_ping_time(vers);
     nuser.protocol   = vers;
     nuser.servseq    = 1;
     nuser.session_id = session_id;
     nuser.tcpver     = tcpversion;

     nuser.lutime = time(NULL);
     db_online_insert(nuser);
     
     v5_send_login_reply(int_pack, userinfo);

     move_user_online(nuser);
     LOG_USR(0, ("User %lu from %s:%ld moved online.\n", nuser.uin, 
                            inet_ntoa(nuser.ip), nuser.udp_port));

  } else
  {
     v5_send_busy(int_pack);
     LOG_USR(0, ("User %lu from %s:%d request login but allready online.\n", 
                 uin_num, inet_ntoa(int_pack.from_ip), int_pack.from_port)); 
  } 
}

/**************************************************************************/
/* Contact list hanlder							  */
/**************************************************************************/
void v5_process_contact()
{
 unsigned short pvers, pcomm, seq1, seq2, i;
 unsigned long  uin_num, temp_stamp, *contact, session_id;
 int count; char ccount;
 struct online_user user, auser;
 PGresult *res;
 fstring dbcomm_str;
 int on_cnt;

 int_pack.reset();
 int_pack  >> pvers   >> temp_stamp
	   >> uin_num >> session_id
	   >> pcomm   >> seq1
           >> seq2    >> temp_stamp
	   >> ccount;
 
 count = (int)ccount;
 contact = (unsigned long *)malloc(sizeof(unsigned long)*count);
 
 for (i=0;i<count;i++) int_pack >> contact[i];

 if (db_online_lookup(uin_num, user) == 0)
 {
    if (ipToIcq(user.ip) == ipToIcq(int_pack.from_ip))
    {
       v5_send_ack(int_pack);
       /* first of all we need ready contact-list in database */
       db_contact_insert(uin_num, count, contact, NORMAL_CONTACT);
       
       /* now we'll try to get online users from our contact */
       slprintf(dbcomm_str, sizeof(dbcomm_str)-1, 
       "SELECT uin,iadr,tprt,stat,sess_id,tcpver FROM online_users 
        WHERE uin IN (SELECT tuin FROM online_contacts WHERE ouin=%lu)", 
	uin_num);

       res = PQexec(users_dbconn, dbcomm_str);
       if (PQresultStatus(res) != PGRES_TUPLES_OK)
       {
          LOG_SYS(0, ("SQL Command failed...\n"));
          LOG_SYS(0, ("%s", PQresultErrorMessage(res)));
          PQclear(res);
          exit(-1);
       }

       if (PQnfields(res) != 6) 
       {
          LOG_SYS(0, ("Corrypted table structure in db: 
	               \"%s\"\n", lp_db_users()));
          exit(-1);
       }
   
       on_cnt = PQntuples(res);
       db_online_sseq_open(user);
       
       for (i=0;i<on_cnt;i++) 
       {
         /* fill auser structure */
	 auser.uin	  = atol(PQgetvalue(res, i, 0));
         auser.ip	  = icqToIp(atol(PQgetvalue(res, i, 1)));
         auser.tcp_port	  = atol(PQgetvalue(res, i, 2)); 
         auser.status	  = atol(PQgetvalue(res, i, 3));
	 auser.session_id = atol(PQgetvalue(res, i, 4));
	 auser.tcpver	  = atol(PQgetvalue(res, i, 5));

	 /* send packet to user */
         v5_send_user_online(user, auser);
       }
       
       PQclear(res);
       v5_send_end_contact(user);
       db_online_sseq_close(user, on_cnt+1);
       
    } else {
    
       LOG_ALARM(0, ("Spoofed contact list from %s:%d (%lu)\n", 
       inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num));
    }   
 } else {
  
   v5_send_not_connected(int_pack);

 }
 delete contact;
}


/**************************************************************************/
/* Used to send packets, that should be confirmed with ack 		  */
/**************************************************************************/
void v5_send_indirect(Packet &pack, unsigned long to_uin, 
		      unsigned short seq1)
{
    unsigned long temp_stamp;

    temp_stamp = generate_ask_number(to_uin, (unsigned long)seq1, 0); 
    event_send_packet(pack, to_uin, v5_retries, v5_timeout, temp_stamp);
}


/**************************************************************************/
/* Used to init all proto variables and data structures 		  */
/**************************************************************************/
void v5_proto_init()
{

 v5_timeout = lp_v5_timeout();
 v5_retries = lp_v5_retries();
  
}

/**************************************************************************/
/* ack packet handler							  */
/**************************************************************************/
void v5_process_ack()
{
  unsigned short pvers, pcomm, seq1, seq2;
  unsigned long uin_num, temp_stamp, session_id;

  int_pack.reset();
  int_pack  >> pvers   >> temp_stamp
	    >> uin_num >> session_id
	    >> pcomm   >> seq1
            >> seq2    >> temp_stamp;
    
  temp_stamp = generate_ask_number(uin_num, seq1, seq2);
 
  DEBUG(150, ("GOT ASK: ver:%d, comm:%d, seq1:%d, seq2:%d, uin:%lu, stamp:%lu\n",
	                 pvers, pcomm, seq1, seq2, uin_num, temp_stamp));
 
  process_ask_event(uin_num, temp_stamp);
}


/**************************************************************************/
/* "Text message" packet handler					  */
/**************************************************************************/
void v5_process_logoff()
{
  unsigned short pvers, pcomm, seq1, seq2, i, plen;
  unsigned long  uin_num, temp_stamp, session_id;
  char bmessage[20];

 int_pack.reset();
 int_pack  >> pvers   >> temp_stamp
	   >> uin_num >> session_id
	   >> pcomm   >> seq1
           >> seq2    >> temp_stamp;

 int_pack >> plen; if (plen > 20 ) plen = 20;
 for (i=0; i<plen; i++) int_pack >> bmessage[i];

 DEBUG(50, ("Bmessage: %s\n", bmessage));
 
 if (strequal(bmessage,"B_USER_DISCONNECTED"))
 {
    struct online_user temp_user;
    unsigned long uptime;
    
    if (db_online_lookup(uin_num, temp_user) == 0)
    {
      if ((ipToIcq(temp_user.ip) == ipToIcq(int_pack.from_ip)) &&
          (temp_user.session_id == session_id))
      { 
         move_user_offline(uin_num);
	 uptime = (unsigned long)(time(NULL) - longToTime(temp_user.uptime));
	 
	 LOG_USR(0, ("User %lu moved offline. Online time was: %lu seconds\n", uin_num, uptime));
      }
      else 
      { 
        /* All mirabilis clients have practic to send this packet */
	/* in disconnect even they are not connected */
	
	/*  LOG_ALARM(0, ("Someone from %s:%d trying to disconnect user %d\n", 
                    inet_ntoa(int_pack.from_ip), int_pack.from_port, uin_num)); */
      }
    }
    else 
    {
      v5_send_not_connected(int_pack);
    }
 }
}
