/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"

static int fd;
struct rlimit flimit;

/**************************************************************************/
/* This proc write pid file to lp_pid_path and lock it			  */
/**************************************************************************/
BOOL write_pid() 
{
    close(open(lp_pid_path(), O_CREAT|O_EXCL, S_IRUSR|S_IWUSR|S_IRGRP));
    if ( (pidFP = fopen(lp_pid_path(), "r+")) == NULL)
    {
        DEBUG(0,("ERROR: Can't write pid file [%s] (%d %s)\n", lp_pid_path(), errno, strerror(errno)));
        return(False);
    }

    if (Lock(pidFP) != 0)
    {
        DEBUG(0,("ERROR: Can't get lock on pid file [%s] (%d %s)\n", 
		 lp_pid_path(), errno, strerror(errno)));
        return(False);
    }

    rewind(pidFP);
    (void)fprintf(pidFP, "%ld", (long int)getpid());
    (void)fflush(pidFP);
    DEBUG(3,("Wrote pid file...\n"));

    return(True);
}


/**************************************************************************/
/* Go to daemon mode							  */
/**************************************************************************/
int become_daemon(void)
{
    if (sys_fork()) _exit(0);
	
    (void)setpgid(0, 0);
    LOG_SYS(4, ("Init: Create new process group\n"));
    
    fd = open("/dev/tty", O_RDWR);
    (void)ioctl(fd, TIOCNOTTY, 0);
    (void)close(fd); 
    LOG_SYS(4, ("Init: Detach from control terminal\n"));
    
    // we don't need signals from terminal
    
    if (getppid() != 1)	       // check if we are running by init
    {
	signal(SIGTTOU, SIG_IGN);
	signal(SIGTTIN, SIG_IGN);
	signal(SIGTSTP, SIG_IGN);
    }
    
    close(0); close(1); close(2);   // close stdio streams
    
    if (write_pid() != True) 
    {
       LOG_SYS(0, ("ERROR: Can't write lock file [another process running ?]...\n"));exit(-1);
    }  else {
       LOG_SYS(0, ("Init: Wrote & lock server pidfile.\n"));
    }

    return(0);

}


/**************************************************************************/
/* Init signals handlers for socket processor				  */
/**************************************************************************/
BOOL Signals_Init(void)
{
    if ( (rsignal(SIGHUP,  (void(*)(int))mySIGHUPHandler)  == SIG_ERR)  ||
         (rsignal(SIGINT,  (void(*)(int))mySIGINTHandler)  == SIG_ERR)  ||
         (rsignal(SIGQUIT, (void(*)(int))mySIGINTHandler)  == SIG_ERR)  ||
         (rsignal(SIGTERM, (void(*)(int))mySIGINTHandler)  == SIG_ERR)  ||
	 (rsignal(SIGCHLD, (void(*)(int))mySIGCHLDHandler) == SIG_ERR) )
    {
        return(False);
    }
    return(True);
}


/**************************************************************************/
/* Init signals handlers for childs					  */
/**************************************************************************/
void childSignals_Init(void)
{
    rsignal(SIGHUP,  (void(*)(int))childSIGINTHandler);
    rsignal(SIGALRM, (void(*)(int))childSIGINTHandler);
    rsignal(SIGINT,  (void(*)(int))childSIGINTHandler);
    rsignal(SIGQUIT, (void(*)(int))childSIGINTHandler);
    rsignal(SIGTERM, (void(*)(int))childSIGINTHandler);
    rsignal(SIGCHLD, (void(*)(int))childSIGINTHandler);
}

