/**************************************************************************/
/*									  */
/* Copyright (c) 2000 by Alexandr V.Shutko, Khabarovsk, Russia		  */
/* All rights reserved.							  */
/*									  */
/* Redistribution and use in source and binary forms, with or without	  */
/* modification, are permitted provided that the following conditions	  */
/* are met:								  */
/* 1. Redistributions of source code must retain the above copyright	  */
/*    notice, this list of conditions and the following disclaimer.	  */
/* 2. Redistributions in binary form must reproduce the above copyright	  */
/*    notice, this list of conditions and the following disclaimer in 	  */
/*    the documentation and/or other materials provided with the 	  */
/*    distribution.							  */
/*									  */
/* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 	  */
/* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS  */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 	  */
/* OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT   */
/* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 	  */
/* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  */
/* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE   */
/* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 	  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.			  */
/*									  */
/**************************************************************************/

#include "includes.h"

/**************************************************************************/
/* Func: reliable signal() function 					  */
/**************************************************************************/
void (*rsignal(int signo, void (*hndlr)(int)))(int)
{
  struct sigaction act, oact;
  
  act.sa_handler = hndlr;
  sigemptyset(&act.sa_mask);
  act.sa_flags = SA_RESTART;
  
  if (sigaction(signo, &act, &oact) < 0) return (SIG_ERR);
  
  return (oact.sa_handler);
}

/**************************************************************************/
/* Func: reliable signal blocking function 				  */
/**************************************************************************/
void block_signal(int signo)
{
  sigset_t signal_set;
  sigset_t old_signal_set;
  
  sigemptyset(&signal_set);
  sigaddset(&signal_set, signo);
  sigprocmask(SIG_BLOCK, &signal_set, &old_signal_set);
 
}

/**************************************************************************/
/* Func: reliable signal unblocking function 				  */
/**************************************************************************/
void unblock_signal(int signo)
{
  sigset_t signal_set;
  sigset_t old_signal_set;
  
  sigemptyset(&signal_set);
  sigaddset(&signal_set, signo);
  sigprocmask(SIG_UNBLOCK, &signal_set, &old_signal_set);
 
}

/**************************************************************************/
/* Func: SIGHUP handler							  */
/**************************************************************************/
RETSIGTYPE mySIGHUPHandler(int param)
{
    BOOL ret;

    LOG_SYS(0,("SIGHUP catched: Reloading configuration...\n"));
    if (lp_file_list_changed()) 
    {
       exit_ok = False;
       ret = lp_load(ICQ_CONFIG_FILE, False, False, True);
       reopen_logs();
       old_interface = bind_interface;
       init_bindinterface();
       udpserver_restart(lp_udp_port());
       killpg(0, SIGINT);
       exit_ok = True;
    }
    
    return;
}

/**************************************************************************/
/* Func: parent SIGINT handler						  */
/**************************************************************************/
RETSIGTYPE mySIGINTHandler(int param)
{   
    if (exit_ok)
    {
       LOG_SYS(0, ("SIGINT catched: Shutting down...\n"));
       signal(SIGINT, SIG_IGN);
       exit(0);
    }
}

/**************************************************************************/
/* Func: child SIGINT handler						  */
/**************************************************************************/
RETSIGTYPE childSIGINTHandler(int param)
{   
       DEBUG(10, ("[%d] SIGINT catched: Shutting down...\n", getpid()));
       PQfinish(users_dbconn);
       
       exit(0);
}


RETSIGTYPE mySIGALRMHandler(int param)
{
    return;
}

/**************************************************************************/
/* Function    : SIGCHLD handler					  */
/**************************************************************************/
RETSIGTYPE mySIGCHLDHandler(int param)
{
    pid_t              cpid = 1;
    int                status;
    
    while(cpid > 0)
    {
        cpid = waitpid(0, &status, WNOHANG);
        if (cpid < 0) { return; }
        else if (cpid == 0) return;

      if (!no_deth_messages)
      {
        if (WIFEXITED(status))
        {
          LOG_SYS(50, ("SIGCHLD: child exited: pid=[%ld], ec=[%d]%s\n", (unsigned long)cpid,
                 WEXITSTATUS(status), (WCOREDUMP(status) ? " (core dumped)" : "")));
        }
        else if (WIFSIGNALED(status))
        {
            LOG_SYS(50, ("SIGCHLD: child exited via signal: pid=[%ld], signal=[%d], ec=[%d]%s\n",
                   (unsigned long)cpid, WTERMSIG(status), WEXITSTATUS(status), 
		   (WCOREDUMP(status) ? " (core dumped)" : "")));
        }
        else
        {
            LOG_SYS(50, ("SIGCHLD: child died with no status information pid=[%ld]\n",
                   (unsigned long)cpid));
        }


        if (child_lookup(cpid) != ROLE_BUSY)
        {
	  LOG_SYS(50, ("SIGCHLD: child role was [%d], new process will start.\n", child_lookup(cpid)));
	  childs_check = 1;
        }	
      }
	child_delete(cpid);
    }
}
